@@ -18,31 +18,54 @@
  *               as published by the Free Software Foundation; either version
  *               2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@gmail.com>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #include "config.h"
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE /* to make O_CLOEXEC available */
-#endif
-
+#include <errno.h>
 #include <fcntl.h>
+#include <unistd.h>
+#include <sys/wait.h>
 #include <stdint.h>
 #include <stdbool.h>
+#include <sys/stat.h>
 
 #ifndef O_CLOEXEC	/* Since Linux 2.6.23 and glibc 2.7 */
 #define O_CLOEXEC 0	/* It doesn't really matter if O_CLOEXEC isn't set here */
 #endif
 
 #include "ipvswrapper.h"
+#include "global_data.h"
 #include "list.h"
 #include "utils.h"
-#include "memory.h"
 #include "logger.h"
+#include "libipvs.h"
+#include "main.h"
 
 static bool no_ipvs = false;
 
+static const char * __attribute__((pure))
+ipvs_cmd_str(int cmd)
+{
+	switch (cmd)
+	{
+		switch_define_str(IP_VS_SO_SET_ADD);
+		switch_define_str(IP_VS_SO_SET_ADDDEST);
+		switch_define_str(IP_VS_SO_SET_DEL);
+		switch_define_str(IP_VS_SO_SET_DELDEST);
+		switch_define_str(IP_VS_SO_SET_EDIT);
+		switch_define_str(IP_VS_SO_SET_EDITDEST);
+		switch_define_str(IP_VS_SO_SET_FLUSH);
+		switch_define_str(IP_VS_SO_SET_STARTDAEMON);
+		switch_define_str(IP_VS_SO_SET_STOPDAEMON);
+		switch_define_str(IP_VS_SO_SET_TIMEOUT);
+		switch_define_str(IP_VS_SO_SET_ZERO);
+	}
+
+	return "(unknown)";
+}
+
 /*
  * Utility functions coming from Wensong code
  */
@@ -53,6 +76,7 @@ get_modprobe(void)
 	int procfile;
 	char *ret;
 	ssize_t count;
+	struct stat buf;
 
 	ret = MALLOC(PATH_MAX);
 	if (!ret)
@@ -64,16 +88,22 @@ get_modprobe(void)
 		return NULL;
 	}
 
-	count = read(procfile, ret, PATH_MAX);
+	count = read(procfile, ret, PATH_MAX - 1);
+	ret[PATH_MAX - 1] = '\0';
 	close(procfile);
 
-	if (count > 0 && count < PATH_MAX)
+	if (count > 0 && count < PATH_MAX - 1)
 	{
 		if (ret[count - 1] == '\n')
 			ret[count - 1] = '\0';
 		else
 			ret[count] = '\0';
-		return ret;
+
+		/* Check it is a regular file, with a execute bit set */
+		if (!stat(ret, &buf) &&
+		    S_ISREG(buf.st_mode) &&
+		    (buf.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)))
+			return ret;
 	}
 
 	FREE(ret);
@@ -84,12 +114,13 @@ get_modprobe(void)
 static bool
 modprobe_ipvs(void)
 {
-	char *argv[] = { "/sbin/modprobe", "-s", "--", "ip_vs", NULL };
+	const char *argv[] = { "/sbin/modprobe", "-s", "--", "ip_vs", NULL };
 	int child;
 	int status;
 	int rc;
 	char *modprobe = get_modprobe();
 	struct sigaction act, old_act;
+	union non_const_args args;
 
 	if (modprobe)
 		argv[0] = modprobe;
@@ -100,8 +131,15 @@ modprobe_ipvs(void)
 
 	sigaction ( SIGCHLD, &act, &old_act);
 
+#ifdef ENABLE_LOG_TO_FILE
+	if (log_file_name)
+		flush_log_file();
+#endif
+
 	if (!(child = fork())) {
-		execv(argv[0], argv);
+		args.args = argv;
+		/* coverity[tainted_string] */
+		execv(argv[0], args.execve_args);
 		exit(1);
 	}
 
@@ -124,14 +162,13 @@ modprobe_ipvs(void)
 	return false;
 }
 /* fetch virtual server group from group name */
-virtual_server_group_t *
-ipvs_get_group_by_name(char *gname, list l)
+virtual_server_group_t * __attribute__ ((pure))
+ipvs_get_group_by_name(const char *gname, list l)
 {
 	element e;
 	virtual_server_group_t *vsg;
 
-	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
-		vsg = ELEMENT_DATA(e);
+	LIST_FOREACH(l, vsg, e) {
 		if (!strcmp(vsg->gname, gname))
 			return vsg;
 	}
@@ -142,7 +179,7 @@ ipvs_get_group_by_name(char *gname, list l)
 int
 ipvs_start(void)
 {
-	log_message(LOG_DEBUG, "Initializing ipvs");
+	log_message(LOG_DEBUG, "%snitializing ipvs", reload ? "Rei" : "I");
 	/* Initialize IPVS module */
 	if (ipvs_init()) {
 		if (modprobe_ipvs() || ipvs_init()) {
@@ -177,7 +214,8 @@ ipvs_set_timeouts(int tcp_timeout, int tcpfin_timeout, int udp_timeout)
 	to.tcp_fin_timeout = tcpfin_timeout;
 	to.udp_timeout = udp_timeout;
 
-	ipvs_set_timeout(&to);
+	if (ipvs_set_timeout(&to))
+		log_message(LOG_INFO, "Failed to set ipvs timeouts");
 }
 
 /* Send user rules to IPVS module */
@@ -235,7 +273,7 @@ ipvs_talk(int cmd, ipvs_service_t *srule, ipvs_dest_t *drule, ipvs_daemon_t *dae
 		else if (errno == ENOENT &&
 			(cmd == IP_VS_SO_SET_DEL || cmd == IP_VS_SO_SET_DELDEST))
 			result = 0;
-		log_message(LOG_INFO, "IPVS (cmd %d, errno %d): %s", cmd, errno, ipvs_strerror(errno));
+		log_message(LOG_INFO, "IPVS cmd %s(%d) error: %s(%d)", ipvs_cmd_str(cmd), cmd, ipvs_strerror(errno), errno);
 	}
 	return result;
 }
@@ -254,7 +292,7 @@ ipvs_syncd_cmd(int cmd, const struct lvs_syncd_config *config, int state, bool i
 	if (config) {
 		daemonrule.syncid = (int)config->syncid;
 		if (!ignore_interface)
-			strncpy(daemonrule.mcast_ifn, config->ifname, IP_VS_IFNAME_MAXLEN);
+			strcpy_safe(daemonrule.mcast_ifn, config->ifname);
 #ifdef _HAVE_IPVS_SYNCD_ATTRIBUTES_
 		if (cmd == IPVS_STARTDAEMON) {
 			if (config->sync_maxlen)
@@ -265,11 +303,11 @@ ipvs_syncd_cmd(int cmd, const struct lvs_syncd_config *config, int state, bool i
 				daemonrule.mcast_ttl = config->mcast_ttl;
 			if (config->mcast_group.ss_family == AF_INET) {
 				daemonrule.mcast_af = AF_INET;
-				daemonrule.mcast_group.ip = ((struct sockaddr_in *)&config->mcast_group)->sin_addr.s_addr;
+				daemonrule.mcast_group.ip = ((const struct sockaddr_in *)&config->mcast_group)->sin_addr.s_addr;
 			}
 			else if (config->mcast_group.ss_family == AF_INET6) {
 				daemonrule.mcast_af = AF_INET6;
-				memcpy(&daemonrule.mcast_group.in6, &((struct sockaddr_in6 *)&config->mcast_group)->sin6_addr, sizeof(daemonrule.mcast_group.in6));
+				memcpy(&daemonrule.mcast_group.in6, &((const struct sockaddr_in6 *)&config->mcast_group)->sin6_addr, sizeof(daemonrule.mcast_group.in6));
 			}
 		}
 #endif
@@ -317,7 +355,7 @@ ipvs_group_range_cmd(int cmd, ipvs_service_t *srule, ipvs_dest_t *drule, virtual
 static bool
 is_vsge_alive(virtual_server_group_entry_t *vsge, virtual_server_t *vs)
 {
-	if (vsge->vfwmark) {
+	if (vsge->is_fwmark) {
 		if (vs->af == AF_INET)
 			return !!vsge->fwm4_alive;
 		else
@@ -332,11 +370,11 @@ is_vsge_alive(virtual_server_group_entry_t *vsge, virtual_server_t *vs)
 }
 
 static void
-update_vsge_alive_count(virtual_server_group_entry_t *vsge, virtual_server_t *vs, bool up)
+update_vsge_alive_count(virtual_server_group_entry_t *vsge, const virtual_server_t *vs, bool up)
 {
 	unsigned *alive_p;
 
-	if (vsge->vfwmark) {
+	if (vsge->is_fwmark) {
 		if (vs->af == AF_INET)
 			alive_p = &vsge->fwm4_alive;
 		else
@@ -356,13 +394,13 @@ update_vsge_alive_count(virtual_server_group_entry_t *vsge, virtual_server_t *vs
 }
 
 static void
-set_vsge_alive(virtual_server_group_entry_t *vsge, virtual_server_t *vs)
+set_vsge_alive(virtual_server_group_entry_t *vsge, const virtual_server_t *vs)
 {
 	update_vsge_alive_count(vsge, vs, true);
 }
 
 static void
-unset_vsge_alive(virtual_server_group_entry_t *vsge, virtual_server_t *vs)
+unset_vsge_alive(virtual_server_group_entry_t *vsge, const virtual_server_t *vs)
 {
 	update_vsge_alive_count(vsge, vs, false);
 }
@@ -375,11 +413,11 @@ ipvs_change_needed(int cmd, virtual_server_group_entry_t *vsge, virtual_server_t
 	if (cmd == IP_VS_SO_SET_ADD)
 		return !is_vsge_alive(vsge, vs);
 	else if (cmd == IP_VS_SO_SET_DEL) {
-		count = vsge->vfwmark ? (vs->af == AF_INET ? vsge->fwm4_alive : vsge->fwm6_alive) :
+		count = vsge->is_fwmark ? (vs->af == AF_INET ? vsge->fwm4_alive : vsge->fwm6_alive) :
 			vs->service_type == IPPROTO_TCP ? vsge->tcp_alive :
 			vs->service_type == IPPROTO_UDP ? vsge->udp_alive : vsge->sctp_alive;
 
-		return (count == 1);
+		return (count == 0);
 	}
 	else if (cmd == IP_VS_SO_SET_ADDDEST)
 		return !rs->alive;
@@ -392,9 +430,9 @@ ipvs_change_needed(int cmd, virtual_server_group_entry_t *vsge, virtual_server_t
 static void
 ipvs_set_vsge_alive_state(int cmd, virtual_server_group_entry_t *vsge, virtual_server_t *vs)
 {
-	if (cmd == IP_VS_SO_SET_ADD)
+	if (cmd == IP_VS_SO_SET_ADDDEST)
 		set_vsge_alive(vsge, vs);
-	else if (cmd == IP_VS_SO_SET_DEL)
+	else if (cmd == IP_VS_SO_SET_DELDEST)
 		unset_vsge_alive(vsge, vs);
 }
 
@@ -409,43 +447,48 @@ ipvs_group_cmd(int cmd, ipvs_service_t *srule, ipvs_dest_t *drule, virtual_serve
 	if (!vsg)
 		return 0;
 
-	/* visit addr_ip list */
-	for (e = LIST_HEAD(vsg->addr_range); e; ELEMENT_NEXT(e)) {
-		vsg_entry = ELEMENT_DATA(e);
-
+	/* visit addr_range list */
+	LIST_FOREACH(vsg->addr_range, vsg_entry, e) {
+		if (cmd == IP_VS_SO_SET_ADD && reload && vsg_entry->reloaded)
+			continue;
 		if (ipvs_change_needed(cmd, vsg_entry, vs, rs)) {
 			srule->user.port = inet_sockaddrport(&vsg_entry->addr);
-
-			if (vsg_entry->range) {
-				if (ipvs_group_range_cmd(cmd, srule, drule, vsg_entry))
-					return -1;
-			} else {
-				if (vsg_entry->addr.ss_family == AF_INET6)
-					inet_sockaddrip6(&vsg_entry->addr, &srule->nf_addr.in6);
+			if (rs) {
+				if (rs->forwarding_method != IP_VS_CONN_F_MASQ)
+					drule->user.port = srule->user.port;
 				else
-					srule->nf_addr.ip = inet_sockaddrip4(&vsg_entry->addr);
-
-				/* Talk to the IPVS channel */
-				if (ipvs_talk(cmd, srule, drule, NULL, false))
-					return -1;
+					drule->user.port = inet_sockaddrport(&rs->addr);
 			}
-			ipvs_set_vsge_alive_state(cmd, vsg_entry, vs);
+
+			if (ipvs_group_range_cmd(cmd, srule, drule, vsg_entry))
+				return -1;
 		}
+
+		if (cmd == IP_VS_SO_SET_ADDDEST || cmd == IP_VS_SO_SET_DELDEST)
+			ipvs_set_vsge_alive_state(cmd, vsg_entry, vs);
 	}
 
 	/* visit vfwmark list */
 	memset(&srule->nf_addr, 0, sizeof(srule->nf_addr));
 	srule->user.port = 0;
-	for (e = LIST_HEAD(vsg->vfwmark); e; ELEMENT_NEXT(e)) {
-		vsg_entry = ELEMENT_DATA(e);
+	if (rs) {
+		if (rs->forwarding_method != IP_VS_CONN_F_MASQ)
+			drule->user.port = 0;
+		else
+			drule->user.port = inet_sockaddrport(&rs->addr);
+	}
+	LIST_FOREACH(vsg->vfwmark, vsg_entry, e) {
+		if (cmd == IP_VS_SO_SET_ADD && reload && vsg_entry->reloaded)
+			continue;
+
 		srule->user.fwmark = vsg_entry->vfwmark;
 
 		/* Talk to the IPVS channel */
 		if (ipvs_change_needed(cmd, vsg_entry, vs, rs)) {
 			if (ipvs_talk(cmd, srule, drule, NULL, false))
 				return -1;
-			ipvs_set_vsge_alive_state(cmd, vsg_entry, vs);
 		}
+		ipvs_set_vsge_alive_state(cmd, vsg_entry, vs);
 	}
 
 	return 0;
@@ -458,7 +501,7 @@ ipvs_set_srule(int cmd, ipvs_service_t *srule, virtual_server_t *vs)
 	/* Clean service rule */
 	memset(srule, 0, sizeof(ipvs_service_t));
 
-	strncpy(srule->user.sched_name, vs->sched, IP_VS_SCHEDNAME_MAXLEN);
+	strcpy_safe(srule->user.sched_name, vs->sched);
 	srule->af = vs->af;
 	srule->user.flags = vs->flags;
 	srule->user.netmask = (vs->af == AF_INET6) ? 128 : ((uint32_t) 0xffffffff);
@@ -500,6 +543,13 @@ ipvs_set_drule(int cmd, ipvs_dest_t *drule, real_server_t * rs)
 	drule->user.weight = rs->weight;
 	drule->user.u_threshold = rs->u_threshold;
 	drule->user.l_threshold = rs->l_threshold;
+#ifdef _HAVE_IPVS_TUN_TYPE_
+	drule->tun_type = rs->tun_type;
+	drule->tun_port = rs->tun_port;
+#ifdef _HAVE_IPVS_TUN_CSUM_
+	drule->tun_flags = rs->tun_flags;
+#endif
+#endif
 }
 
 /* Set/Remove a RS from a VS */
@@ -533,17 +583,21 @@ ipvs_cmd(int cmd, virtual_server_t *vs, real_server_t *rs)
 	}
 
 	/* Set vs rule and send to kernel */
-	if (vs->vsgname)
+	if (vs->vsg)
 		return ipvs_group_cmd(cmd, &srule, &drule, vs, rs);
 
 	if (vs->vfwmark) {
 		srule.user.fwmark = vs->vfwmark;
+		if (rs && rs->forwarding_method != IP_VS_CONN_F_MASQ)
+			drule.user.port = 0;
 	} else {
 		if (vs->af == AF_INET6)
 			inet_sockaddrip6(&vs->addr, &srule.nf_addr.in6);
 		else
 			srule.nf_addr.ip = inet_sockaddrip4(&vs->addr);
 		srule.user.port = inet_sockaddrport(&vs->addr);
+		if (rs && rs->forwarding_method != IP_VS_CONN_F_MASQ)
+			drule.user.port = srule.user.port;
 	}
 
 	/* Talk to the IPVS channel */
@@ -560,13 +614,13 @@ ipvs_group_sync_entry(virtual_server_t *vs, virtual_server_group_entry_t *vsge)
 	ipvs_dest_t drule;
 
 	ipvs_set_srule(IP_VS_SO_SET_ADDDEST, &srule, vs);
-	if (!vsge->vfwmark)
+	if (vsge->is_fwmark)
+		srule.user.fwmark = vsge->vfwmark;
+	else
 		srule.user.port = inet_sockaddrport(&vsge->addr);
 
 	/* Process realserver queue */
-	for (e = LIST_HEAD(vs->rs); e; ELEMENT_NEXT(e)) {
-		rs = ELEMENT_DATA(e);
-
+	LIST_FOREACH(vs->rs, rs, e) {
 // ??? What if !quorum_state_up?
 		if (rs->reloaded && (rs->alive || (rs->inhibit && rs->set))) {
 			/* Prepare the IPVS drule */
@@ -574,21 +628,12 @@ ipvs_group_sync_entry(virtual_server_t *vs, virtual_server_group_entry_t *vsge)
 			drule.user.weight = rs->inhibit && !rs->alive ? 0 : rs->weight;
 
 			/* Set vs rule */
-			if (vsge->range)
-				ipvs_group_range_cmd(IP_VS_SO_SET_ADDDEST, &srule, &drule, vsge);
-			else {
-				if (vsge->vfwmark)
-					srule.user.fwmark = vsge->vfwmark;
-				else {
-					if (vsge->addr.ss_family == AF_INET6)
-						inet_sockaddrip6(&vsge->addr, &srule.nf_addr.in6);
-					else
-						srule.nf_addr.ip = inet_sockaddrip4(&vsge->addr);
-				}
-
+			if (vsge->is_fwmark) {
 				/* Talk to the IPVS channel */
 				ipvs_talk(IP_VS_SO_SET_ADDDEST, &srule, &drule, NULL, false);
 			}
+			else
+				ipvs_group_range_cmd(IP_VS_SO_SET_ADDDEST, &srule, &drule, vsge);
 		}
 	}
 }
@@ -604,42 +649,34 @@ ipvs_group_remove_entry(virtual_server_t *vs, virtual_server_group_entry_t *vsge
 
 	/* Prepare target rules */
 	ipvs_set_srule(IP_VS_SO_SET_DELDEST, &srule, vs);
-	if (!vsge->vfwmark)
+	if (vsge->is_fwmark)
+		srule.user.fwmark = vsge->vfwmark;
+	else
 		srule.user.port = inet_sockaddrport(&vsge->addr);
 
 	/* Process realserver queue */
-	for (e = LIST_HEAD(vs->rs); e; ELEMENT_NEXT(e)) {
-		rs = ELEMENT_DATA(e);
-
+	LIST_FOREACH(vs->rs, rs, e) {
 		if (rs->alive) {
 			/* Setting IPVS drule */
 			ipvs_set_drule(IP_VS_SO_SET_DELDEST, &drule, rs);
 
-			/* Set vs rule */
-			if (vsge->range)
-				ipvs_group_range_cmd(IP_VS_SO_SET_DELDEST, &srule, &drule, vsge);
-			else {
-				if (vsge->vfwmark)
-					srule.user.fwmark = vsge->vfwmark;
-				else {
-					if (vsge->addr.ss_family == AF_INET6)
-						inet_sockaddrip6(&vsge->addr, &srule.nf_addr.in6);
-					else
-						srule.nf_addr.ip = inet_sockaddrip4(&vsge->addr);
-				}
-
+			/* Delete rs rule */
+			if (vsge->is_fwmark) {
 				/* Talk to the IPVS channel */
 				ipvs_talk(IP_VS_SO_SET_DELDEST, &srule, &drule, NULL, false);
 			}
+			else
+				ipvs_group_range_cmd(IP_VS_SO_SET_DELDEST, &srule, &drule, vsge);
 		}
 	}
 
-	/* Remove VS entry */
-	if (vsge->range)
-		ipvs_group_range_cmd(IP_VS_SO_SET_DEL, &srule, NULL, vsge);
-	else
-		ipvs_talk(IP_VS_SO_SET_DEL, &srule, NULL, NULL, false);
-	unset_vsge_alive(vsge,vs);
+	/* Remove VS entry if this is the last VS using it */
+	if (!is_vsge_alive(vsge, vs)) {
+		if (vsge->is_fwmark)
+			ipvs_talk(IP_VS_SO_SET_DEL, &srule, NULL, NULL, false);
+		else
+			ipvs_group_range_cmd(IP_VS_SO_SET_DEL, &srule, NULL, vsge);
+	}
 }
 
 #ifdef _WITH_SNMP_CHECKER_
@@ -657,8 +694,8 @@ vsd_equal(real_server_t *rs, struct ip_vs_dest_entry_app *entry)
 					     : (void *)&((struct sockaddr_in6 *)&rs->addr)->sin6_addr))
 		return false;
 
-	if (entry->user.port != (entry->af == AF_INET) ? ((struct sockaddr_in *)&rs->addr)->sin_port
-						       : ((struct sockaddr_in6 *)&rs->addr)->sin6_port)
+	if (entry->user.port != (entry->af == AF_INET ? ((struct sockaddr_in *)&rs->addr)->sin_port
+						      : ((struct sockaddr_in6 *)&rs->addr)->sin6_port))
 		return false;
 
 	return true;
@@ -707,6 +744,8 @@ ipvs_update_vs_stats(virtual_server_t *vs, uint32_t fwmark, union nf_inet_addr *
 				if (vsd_equal(rs, &dests->user.entrytable[i]))
 					break;
 			}
+			if (!e)
+				rs = NULL;
 		}
 
 		if (rs) {
@@ -741,11 +780,11 @@ ipvs_update_stats(virtual_server_t *vs)
 	union nf_inet_addr nfaddr;
 	unsigned i;
 	real_server_t *rs;
-	time_t time_now = time(NULL);
+	time_t cur_time = time(NULL);
 
-	if (time_now - vs->lastupdated < STATS_REFRESH)
+	if (cur_time - vs->lastupdated < STATS_REFRESH)
 		return;
-	vs->lastupdated = time_now;
+	vs->lastupdated = cur_time;
 
 	/* Reset stats */
 	memset(&vs->stats, 0, sizeof(vs->stats));
@@ -761,7 +800,7 @@ ipvs_update_stats(virtual_server_t *vs)
 	}
 
 	/* Update the stats */
-	if (vs->vsgname) {
+	if (vs->vsg) {
 		for (ge = LIST_HEAD(vs->vsg->vfwmark); ge; ELEMENT_NEXT(ge)) {
 			vsg_entry = ELEMENT_DATA(ge);
 			ipvs_update_vs_stats(vs, vsg_entry->vfwmark, &nfaddr, 0);
