@@ -36,9 +36,12 @@
 #include "vrrp.h"
 #include "vrrp_ipaddress.h"
 #endif
-#if HAVE_DECL_RLIMIT_RTTIME == 1
+#ifdef _HAVE_SCHED_RT_
 #include "process.h"
 #endif
+#ifdef _WITH_FIREWALL_
+#include "vrrp_firewall.h"
+#endif
 
 /* global vars */
 data_t *global_data = NULL;
@@ -46,13 +49,12 @@ data_t *old_global_data = NULL;
 
 /* Default settings */
 static void
-set_default_router_id(data_t *data, char *new_id)
+set_default_router_id(data_t *data, const char *new_id)
 {
 	if (!new_id || !new_id[0])
 		return;
 
-	data->router_id = MALLOC(strlen(new_id)+1);
-	strcpy(data->router_id, new_id);
+	data->router_id = STRDUP(new_id);
 }
 
 static void
@@ -60,7 +62,7 @@ set_default_email_from(data_t * data, const char *hostname)
 {
 	struct passwd *pwd = NULL;
 	size_t len;
-
+	char *str;
 	if (!hostname || !hostname[0])
 		return;
 
@@ -69,11 +71,11 @@ set_default_email_from(data_t * data, const char *hostname)
 		return;
 
 	len = strlen(hostname) + strlen(pwd->pw_name) + 2;
-	data->email_from = MALLOC(len);
+	data->email_from = str = MALLOC(len);
 	if (!data->email_from)
 		return;
 
-	snprintf(data->email_from, len, "%s@%s", pwd->pw_name, hostname);
+	snprintf(str, len, "%s@%s", pwd->pw_name, hostname);
 }
 
 static void
@@ -86,7 +88,9 @@ set_default_smtp_connection_timeout(data_t * data)
 static void
 set_default_mcast_group(data_t * data)
 {
+	/* coverity[check_return] */
 	inet_stosockaddr(INADDR_VRRP_GROUP, 0, (struct sockaddr_storage *)&data->vrrp_mcast_group4);
+	/* coverity[check_return] */
 	inet_stosockaddr(INADDR6_VRRP_GROUP, 0, (struct sockaddr_storage *)&data->vrrp_mcast_group6);
 }
 
@@ -102,16 +106,18 @@ set_vrrp_defaults(data_t * data)
 	data->vrrp_lower_prio_no_advert = false;
 	data->vrrp_higher_prio_send_advert = false;
 	data->vrrp_version = VRRP_VERSION_2;
-	strcpy(data->vrrp_iptables_inchain, "INPUT");
 #ifdef _HAVE_LIBIPSET_
 	data->using_ipsets = true;
-	strcpy(data->vrrp_ipset_address, "keepalived");
-	strcpy(data->vrrp_ipset_address6, "keepalived6");
-	strcpy(data->vrrp_ipset_address_iface6, "keepalived_if6");
+	strcpy(data->vrrp_ipset_address, DEFAULT_IPSET_NAME);
+	strcpy(data->vrrp_ipset_address6, DEFAULT_IPSET_NAME "6");
+	strcpy(data->vrrp_ipset_address_iface6, DEFAULT_IPSET_NAME "if6");
 #endif
 	data->vrrp_check_unicast_src = false;
 	data->vrrp_skip_check_adv_addr = false;
 	data->vrrp_strict = false;
+#ifdef _WITH_NFTABLES_
+	data->vrrp_nf_chain_priority = -1;
+#endif
 }
 #endif
 
@@ -122,22 +128,16 @@ free_email(void *data)
 	FREE(data);
 }
 static void
-dump_email(FILE *fp, void *data)
+dump_email(FILE *fp, const void *data)
 {
-	char *addr = data;
-	conf_write(fp, " Email notification = %s", addr);
+	const char *addr = data;
+	conf_write(fp, "   %s", addr);
 }
 
 void
-alloc_email(char *addr)
+alloc_email(const char *addr)
 {
-	size_t size = strlen(addr);
-	char *new;
-
-	new = (char *) MALLOC(size + 1);
-	memcpy(new, addr, size + 1);
-
-	list_add(global_data->email, new);
+	list_add(global_data->email, STRDUP(addr));
 }
 
 /* data facility functions */
@@ -149,7 +149,7 @@ alloc_global_data(void)
 	if (global_data)
 		return global_data;
 
-	new = (data_t *) MALLOC(sizeof(data_t));
+	new = MALLOC(sizeof(data_t));
 	new->email = alloc_list(free_email, dump_email);
 	new->smtp_alert = -1;
 #ifdef _WITH_VRRP_
@@ -184,7 +184,7 @@ alloc_global_data(void)
 #endif
 
 #ifdef _WITH_SNMP_
-	if (snmp) {
+	if (snmp_option) {
 #ifdef _WITH_SNMP_VRRP_
 		new->enable_snmp_vrrp = true;
 #endif
@@ -199,10 +199,8 @@ alloc_global_data(void)
 #endif
 	}
 
-	if (snmp_socket) {
-		new->snmp_socket = MALLOC(strlen(snmp_socket + 1));
-		strcpy(new->snmp_socket, snmp_socket);
-	}
+	if (snmp_socket)
+		new->snmp_socket = STRDUP(snmp_socket);
 #endif
 
 #ifdef _WITH_LVS_
@@ -218,17 +216,23 @@ alloc_global_data(void)
 }
 
 void
-init_global_data(data_t * data, data_t *old_global_data)
+init_global_data(data_t * data, data_t *prev_global_data, bool copy_network_namespace)
 {
 	/* If this is a reload and we are running in a network namespace,
 	 * we may not be able to get local_name, so preserve it */
-	char unknown_name[] = "[unknown]";
+	const char unknown_name[] = "[unknown]";
 
 	/* If we are running in a network namespace, we may not be
 	 * able to get our local name now, so re-use original */
-	if (old_global_data) {
-		data->local_name = old_global_data->local_name;
-		old_global_data->local_name = NULL;
+	if (prev_global_data) {
+		data->local_name = prev_global_data->local_name;
+		prev_global_data->local_name = NULL;
+
+		if (copy_network_namespace) {
+			FREE_CONST_PTR(data->network_namespace);
+			data->network_namespace = prev_global_data->network_namespace;
+			prev_global_data->network_namespace = NULL;
+		}
 	}
 
 	if (!data->local_name &&
@@ -240,10 +244,8 @@ init_global_data(data_t * data, data_t *old_global_data)
 
 		/* If for some reason get_local_name() fails, we need to have
 		 * some string in local_name, otherwise keepalived can segfault */
-		if (!data->local_name) {
-			data->local_name = MALLOC(sizeof(unknown_name));
-			strcpy(data->local_name, unknown_name);
-		}
+		if (!data->local_name)
+			data->local_name = STRDUP(unknown_name);
 	}
 
 	if (!data->router_id)
@@ -253,12 +255,12 @@ init_global_data(data_t * data, data_t *old_global_data)
 		if (!data->smtp_connection_to)
 			set_default_smtp_connection_timeout(data);
 
-		if (strcmp(data->local_name, unknown_name)) {
+		if (data->local_name && strcmp(data->local_name, unknown_name)) {
 			if (!data->email_from)
 				set_default_email_from(data, data->local_name);
 
 			if (!data->smtp_helo_name)
-				data->smtp_helo_name = data->local_name;
+				data->smtp_helo_name = STRDUP(data->local_name);
 		}
 	}
 
@@ -273,7 +275,7 @@ init_global_data(data_t * data, data_t *old_global_data)
 	    data->notify_fifo.name && data->vrrp_notify_fifo.name &&
 	    !strcmp(data->notify_fifo.name, data->vrrp_notify_fifo.name)) {
 		log_message(LOG_INFO, "notify FIFO %s has been specified for global and vrrp FIFO - ignoring vrrp FIFO", data->vrrp_notify_fifo.name);
-		FREE_PTR(data->vrrp_notify_fifo.name);
+		FREE_CONST_PTR(data->vrrp_notify_fifo.name);
 		data->vrrp_notify_fifo.name = NULL;
 		free_notify_script(&data->vrrp_notify_fifo.script);
 	}
@@ -288,7 +290,7 @@ init_global_data(data_t * data, data_t *old_global_data)
 		if (data->notify_fifo.name && data->lvs_notify_fifo.name &&
 		    !strcmp(data->notify_fifo.name, data->lvs_notify_fifo.name)) {
 			log_message(LOG_INFO, "notify FIFO %s has been specified for global and LVS FIFO - ignoring LVS FIFO", data->lvs_notify_fifo.name);
-			FREE_PTR(data->lvs_notify_fifo.name);
+			FREE_CONST_PTR(data->lvs_notify_fifo.name);
 			data->lvs_notify_fifo.name = NULL;
 			free_notify_script(&data->lvs_notify_fifo.script);
 		}
@@ -316,30 +318,46 @@ free_global_data(data_t * data)
 
 	free_list(&data->email);
 #if HAVE_DECL_CLONE_NEWNET
-	FREE_PTR(data->network_namespace);
+	FREE_CONST_PTR(data->network_namespace);
 #endif
-	FREE_PTR(data->instance_name);
-	FREE_PTR(data->router_id);
-	FREE_PTR(data->email_from);
-	FREE_PTR(data->smtp_helo_name);
-	FREE_PTR(data->local_name);
-#ifdef _WITH_SNMP_
-	FREE_PTR(data->snmp_socket);
-#endif
-#if defined _WITH_LVS_ && defined _WITH_VRRP_
-	FREE_PTR(data->lvs_syncd.ifname);
-	FREE_PTR(data->lvs_syncd.vrrp_name);
-#endif
-	FREE_PTR(data->notify_fifo.name);
-	free_notify_script(&data->notify_fifo.script);
+	FREE_CONST_PTR(data->instance_name);
+	FREE_CONST_PTR(data->process_name);
 #ifdef _WITH_VRRP_
-	FREE_PTR(data->default_ifname);
-	FREE_PTR(data->vrrp_notify_fifo.name);
-	free_notify_script(&data->vrrp_notify_fifo.script);
+	FREE_CONST_PTR(data->vrrp_process_name);
 #endif
 #ifdef _WITH_LVS_
-	FREE_PTR(data->lvs_notify_fifo.name);
+	FREE_CONST_PTR(data->lvs_process_name);
+#endif
+#ifdef _WITH_BFD_
+	FREE_CONST_PTR(data->bfd_process_name);
+#endif
+	FREE_CONST_PTR(data->router_id);
+	FREE_CONST_PTR(data->email_from);
+	FREE_CONST_PTR(data->smtp_helo_name);
+	FREE_CONST_PTR(data->local_name);
+#ifdef _WITH_SNMP_
+	FREE_CONST_PTR(data->snmp_socket);
+#endif
+#if defined _WITH_LVS_ && defined _WITH_VRRP_
+	FREE_CONST_PTR(data->lvs_syncd.ifname);
+	FREE_CONST_PTR(data->lvs_syncd.vrrp_name);
+#endif
+	FREE_CONST_PTR(data->notify_fifo.name);
+	free_notify_script(&data->notify_fifo.script);
+#ifdef _WITH_VRRP_
+	FREE_CONST_PTR(data->default_ifname);
+	FREE_CONST_PTR(data->vrrp_notify_fifo.name);
+	free_notify_script(&data->vrrp_notify_fifo.script);
+#ifdef _WITH_NFTABLES_
+	FREE_CONST_PTR(data->vrrp_nf_table_name);
+#endif
+#endif
+#ifdef _WITH_LVS_
+	FREE_CONST_PTR(data->lvs_notify_fifo.name);
 	free_notify_script(&data->lvs_notify_fifo.script);
+#endif
+#ifdef _WITH_DBUS_
+	FREE_CONST_PTR(data->dbus_service_name);
 #endif
 	FREE(data);
 }
@@ -347,6 +365,9 @@ free_global_data(data_t * data)
 void
 dump_global_data(FILE *fp, data_t * data)
 {
+#ifdef _HAVE_SCHED_RT_
+	char cpu_str[64];
+#endif
 #ifdef _WITH_VRRP_
 	char buf[64];
 #endif
@@ -361,6 +382,20 @@ dump_global_data(FILE *fp, data_t * data)
 #endif
 	if (data->instance_name)
 		conf_write(fp, " Instance name = %s", data->instance_name);
+	if (data->process_name)
+		conf_write(fp, " Parent process name = %s", data->process_name);
+#ifdef _WITH_VRRP_
+	if (data->vrrp_process_name)
+		conf_write(fp, " VRRP process name = %s", data->vrrp_process_name);
+#endif
+#ifdef _WITH_LVS_
+	if (data->lvs_process_name)
+		conf_write(fp, " LVS process name = %s", data->lvs_process_name);
+#endif
+#ifdef _WITH_BFD_
+	if (data->bfd_process_name)
+		conf_write(fp, " BFD process name = %s", data->bfd_process_name);
+#endif
 	if (data->router_id)
 		conf_write(fp, " Router ID = %s", data->router_id);
 	if (data->smtp_server.ss_family) {
@@ -371,10 +406,11 @@ dump_global_data(FILE *fp, data_t * data)
 		conf_write(fp, " Smtp HELO name = %s" , data->smtp_helo_name);
 	if (data->smtp_connection_to)
 		conf_write(fp, " Smtp server connection timeout = %lu"
-				    , data->smtp_connection_to / TIMER_HZ);
+			     , data->smtp_connection_to / TIMER_HZ);
 	if (data->email_from) {
 		conf_write(fp, " Email notification from = %s"
 				    , data->email_from);
+		conf_write(fp, " Email notification to:");
 		dump_list(fp, data->email);
 	}
 	conf_write(fp, " Default smtp_alert = %s",
@@ -386,6 +422,7 @@ dump_global_data(FILE *fp, data_t * data)
 #ifdef _WITH_LVS_
 	conf_write(fp, " Default smtp_alert_checker = %s",
 			data->smtp_alert_checker == -1 ? "unset" : data->smtp_alert_checker ? "on" : "off");
+	conf_write(fp, " Checkers log all failures = %s", data->checker_log_all_failures ? "true" : "false");
 #endif
 #ifdef _WITH_VRRP_
 	conf_write(fp, " Dynamic interfaces = %s", data->dynamic_interfaces ? "true" : "false");
@@ -427,20 +464,22 @@ dump_global_data(FILE *fp, data_t * data)
 	}
 #endif
 	conf_write(fp, " LVS flush = %s", data->lvs_flush ? "true" : "false");
+	conf_write(fp, " LVS flush on stop = %s", data->lvs_flush_onstop == LVS_FLUSH_FULL ? "full" :
+						  data->lvs_flush_onstop == LVS_FLUSH_VS ? "VS" : "disabled");
 #endif
 	if (data->notify_fifo.name) {
-		conf_write(fp, " Global notify fifo = %s", data->notify_fifo.name);
+		conf_write(fp, " Global notify fifo = %s, uid:gid %u:%u", data->notify_fifo.name, data->notify_fifo.uid, data->notify_fifo.gid);
 		if (data->notify_fifo.script)
-			conf_write(fp, " Global notify fifo script = %s, uid:gid %d:%d",
+			conf_write(fp, " Global notify fifo script = %s, uid:gid %u:%u",
 				    cmd_str(data->notify_fifo.script),
 				    data->notify_fifo.script->uid,
 				    data->notify_fifo.script->gid);
 	}
 #ifdef _WITH_VRRP_
 	if (data->vrrp_notify_fifo.name) {
-		conf_write(fp, " VRRP notify fifo = %s", data->vrrp_notify_fifo.name);
+		conf_write(fp, " VRRP notify fifo = %s, uid:gid %u:%u", data->vrrp_notify_fifo.name, data->vrrp_notify_fifo.uid, data->vrrp_notify_fifo.gid);
 		if (data->vrrp_notify_fifo.script)
-			conf_write(fp, " VRRP notify fifo script = %s, uid:gid %d:%d",
+			conf_write(fp, " VRRP notify fifo script = %s, uid:gid %u:%u",
 				    cmd_str(data->vrrp_notify_fifo.script),
 				    data->vrrp_notify_fifo.script->uid,
 				    data->vrrp_notify_fifo.script->gid);
@@ -448,15 +487,16 @@ dump_global_data(FILE *fp, data_t * data)
 #endif
 #ifdef _WITH_LVS_
 	if (data->lvs_notify_fifo.name) {
-		conf_write(fp, " LVS notify fifo = %s", data->lvs_notify_fifo.name);
+		conf_write(fp, " LVS notify fifo = %s, uid:gid %u:%u", data->lvs_notify_fifo.name, data->lvs_notify_fifo.uid, data->lvs_notify_fifo.gid);
 		if (data->lvs_notify_fifo.script)
-			conf_write(fp, " LVS notify fifo script = %s, uid:gid %d:%d",
+			conf_write(fp, " LVS notify fifo script = %s, uid:gid %u:%u",
 				    cmd_str(data->lvs_notify_fifo.script),
 				    data->lvs_notify_fifo.script->uid,
 				    data->lvs_notify_fifo.script->gid);
 	}
 #endif
 #ifdef _WITH_VRRP_
+	conf_write(fp, " VRRP notify priority changes = %s", data->vrrp_notify_priority_changes ? "true" : "false");
 	if (data->vrrp_mcast_group4.sin_family) {
 		conf_write(fp, " VRRP IPv4 mcast group = %s"
 				    , inet_sockaddrtos((struct sockaddr_storage *)&data->vrrp_mcast_group4));
@@ -468,28 +508,40 @@ dump_global_data(FILE *fp, data_t * data)
 	conf_write(fp, " Gratuitous ARP delay = %u",
 		       data->vrrp_garp_delay/TIMER_HZ);
 	conf_write(fp, " Gratuitous ARP repeat = %u", data->vrrp_garp_rep);
-	conf_write(fp, " Gratuitous ARP refresh timer = %lu",
-		       data->vrrp_garp_refresh.tv_sec);
-	conf_write(fp, " Gratuitous ARP refresh repeat = %d", data->vrrp_garp_refresh_rep);
-	conf_write(fp, " Gratuitous ARP lower priority delay = %d", data->vrrp_garp_lower_prio_delay == PARAMETER_UNSET ? PARAMETER_UNSET : data->vrrp_garp_lower_prio_delay / TIMER_HZ);
-	conf_write(fp, " Gratuitous ARP lower priority repeat = %d", data->vrrp_garp_lower_prio_rep);
+	conf_write(fp, " Gratuitous ARP refresh timer = %ld", data->vrrp_garp_refresh.tv_sec);
+	conf_write(fp, " Gratuitous ARP refresh repeat = %u", data->vrrp_garp_refresh_rep);
+	conf_write(fp, " Gratuitous ARP lower priority delay = %u", data->vrrp_garp_lower_prio_delay == PARAMETER_UNSET ? PARAMETER_UNSET : data->vrrp_garp_lower_prio_delay / TIMER_HZ);
+	conf_write(fp, " Gratuitous ARP lower priority repeat = %u", data->vrrp_garp_lower_prio_rep);
 	conf_write(fp, " Send advert after receive lower priority advert = %s", data->vrrp_lower_prio_no_advert ? "false" : "true");
 	conf_write(fp, " Send advert after receive higher priority advert = %s", data->vrrp_higher_prio_send_advert ? "true" : "false");
-	conf_write(fp, " Gratuitous ARP interval = %d", data->vrrp_garp_interval);
-	conf_write(fp, " Gratuitous NA interval = %d", data->vrrp_gna_interval);
+	conf_write(fp, " Gratuitous ARP interval = %f", data->vrrp_garp_interval / TIMER_HZ_DOUBLE);
+	conf_write(fp, " Gratuitous NA interval = %f", data->vrrp_gna_interval / TIMER_HZ_DOUBLE);
 	conf_write(fp, " VRRP default protocol version = %d", data->vrrp_version);
-	if (data->vrrp_iptables_inchain[0])
+#ifdef _WITH_IPTABLES_
+	if (data->vrrp_iptables_inchain[0]) {
 		conf_write(fp," Iptables input chain = %s", data->vrrp_iptables_inchain);
-	if (data->vrrp_iptables_outchain[0])
-		conf_write(fp," Iptables output chain = %s", data->vrrp_iptables_outchain);
+		if (data->vrrp_iptables_outchain[0])
+			conf_write(fp," Iptables output chain = %s", data->vrrp_iptables_outchain);
 #ifdef _HAVE_LIBIPSET_
-	conf_write(fp, " Using ipsets = %s", data->using_ipsets ? "true" : "false");
-	if (data->vrrp_ipset_address[0])
-		conf_write(fp," ipset IPv4 address set = %s", data->vrrp_ipset_address);
-	if (data->vrrp_ipset_address6[0])
-		conf_write(fp," ipset IPv6 address set = %s", data->vrrp_ipset_address6);
-	if (data->vrrp_ipset_address_iface6[0])
-		conf_write(fp," ipset IPv6 address,iface set = %s", data->vrrp_ipset_address_iface6);
+		conf_write(fp, " Using ipsets = %s", data->using_ipsets ? "true" : "false");
+		if (data->using_ipsets) {
+			if (data->vrrp_ipset_address[0])
+				conf_write(fp," ipset IPv4 address set = %s", data->vrrp_ipset_address);
+			if (data->vrrp_ipset_address6[0])
+				conf_write(fp," ipset IPv6 address set = %s", data->vrrp_ipset_address6);
+			if (data->vrrp_ipset_address_iface6[0])
+				conf_write(fp," ipset IPv6 address,iface set = %s", data->vrrp_ipset_address_iface6);
+		}
+#endif
+	}
+#endif
+#ifdef _WITH_NFTABLES_
+	if (data->vrrp_nf_table_name) {
+		conf_write(fp," nftables table name = %s", data->vrrp_nf_table_name);
+		conf_write(fp," nftables base chain priority = %d", data->vrrp_nf_chain_priority);
+		conf_write(fp," nftables with%s counters", data->vrrp_nf_counters ? "" : "out");
+		conf_write(fp," nftables %suse ifname for link local IPv6", data->vrrp_nf_ifindex ? "don't " : "");
+	}
 #endif
 
 	conf_write(fp, " VRRP check unicast_src = %s", data->vrrp_check_unicast_src ? "true" : "false");
@@ -499,8 +551,12 @@ dump_global_data(FILE *fp, data_t * data)
 	conf_write(fp, " VRRP don't swap = %s", data->vrrp_no_swap ? "true" : "false");
 #ifdef _HAVE_SCHED_RT_
 	conf_write(fp, " VRRP realtime priority = %u", data->vrrp_realtime_priority);
+	if (CPU_COUNT(&data->vrrp_cpu_mask)) {
+		get_process_cpu_affinity_string(&data->vrrp_cpu_mask, cpu_str, 63);
+		conf_write(fp, " VRRP CPU Affinity = %s", cpu_str);
+	}
 #if HAVE_DECL_RLIMIT_RTTIME
-	conf_write(fp, " VRRP realtime limit = %lu", data->vrrp_rlimit_rt);
+	conf_write(fp, " VRRP realtime limit = %" PRI_rlim_t, data->vrrp_rlimit_rt);
 #endif
 #endif
 #endif
@@ -509,8 +565,12 @@ dump_global_data(FILE *fp, data_t * data)
 	conf_write(fp, " Checker don't swap = %s", data->checker_no_swap ? "true" : "false");
 #ifdef _HAVE_SCHED_RT_
 	conf_write(fp, " Checker realtime priority = %u", data->checker_realtime_priority);
+	if (CPU_COUNT(&data->checker_cpu_mask)) {
+		get_process_cpu_affinity_string(&data->checker_cpu_mask, cpu_str, 63);
+		conf_write(fp, " Checker CPU Affinity = %s", cpu_str);
+	}
 #if HAVE_DECL_RLIMIT_RTTIME
-	conf_write(fp, " Checker realtime limit = %lu", data->checker_rlimit_rt);
+	conf_write(fp, " Checker realtime limit = %" PRI_rlim_t, data->checker_rlimit_rt);
 #endif
 #endif
 #endif
@@ -519,8 +579,12 @@ dump_global_data(FILE *fp, data_t * data)
 	conf_write(fp, " BFD don't swap = %s", data->bfd_no_swap ? "true" : "false");
 #ifdef _HAVE_SCHED_RT_
 	conf_write(fp, " BFD realtime priority = %u", data->bfd_realtime_priority);
+	if (CPU_COUNT(&data->bfd_cpu_mask)) {
+		get_process_cpu_affinity_string(&data->bfd_cpu_mask, cpu_str, 63);
+		conf_write(fp, " BFD CPU Affinity = %s", cpu_str);
+	}
 #if HAVE_DECL_RLIMIT_RTTIME
-	conf_write(fp, " BFD realtime limit = %lu", data->bfd_rlimit_rt);
+	conf_write(fp, " BFD realtime limit = %" PRI_rlim_t, data->bfd_rlimit_rt);
 #endif
 #endif
 #endif
@@ -545,20 +609,24 @@ dump_global_data(FILE *fp, data_t * data)
 	conf_write(fp, " DBus service name = %s", data->dbus_service_name ? data->dbus_service_name : "");
 #endif
 	conf_write(fp, " Script security %s", script_security ? "enabled" : "disabled");
-	conf_write(fp, " Default script uid:gid %d:%d", default_script_uid, default_script_gid);
+	conf_write(fp, " Default script uid:gid %u:%u", default_script_uid, default_script_gid);
 #ifdef _WITH_VRRP_
 	conf_write(fp, " vrrp_netlink_cmd_rcv_bufs = %u", global_data->vrrp_netlink_cmd_rcv_bufs);
-	conf_write(fp, " vrrp_netlink_cmd_rcv_bufs_force = %u", global_data->vrrp_netlink_cmd_rcv_bufs_force);
+	conf_write(fp, " vrrp_netlink_cmd_rcv_bufs_force = %d", global_data->vrrp_netlink_cmd_rcv_bufs_force);
 	conf_write(fp, " vrrp_netlink_monitor_rcv_bufs = %u", global_data->vrrp_netlink_monitor_rcv_bufs);
-	conf_write(fp, " vrrp_netlink_monitor_rcv_bufs_force = %u", global_data->vrrp_netlink_monitor_rcv_bufs_force);
+	conf_write(fp, " vrrp_netlink_monitor_rcv_bufs_force = %d", global_data->vrrp_netlink_monitor_rcv_bufs_force);
+#ifdef _WITH_CN_PROC_
+	conf_write(fp, " process_monitor_rcv_bufs = %u", global_data->process_monitor_rcv_bufs);
+	conf_write(fp, " process_monitor_rcv_bufs_force = %d", global_data->process_monitor_rcv_bufs_force);
+#endif
 #endif
 #ifdef _WITH_LVS_
 	conf_write(fp, " lvs_netlink_cmd_rcv_bufs = %u", global_data->lvs_netlink_cmd_rcv_bufs);
-	conf_write(fp, " lvs_netlink_cmd_rcv_bufs_force = %u", global_data->lvs_netlink_cmd_rcv_bufs_force);
+	conf_write(fp, " lvs_netlink_cmd_rcv_bufs_force = %d", global_data->lvs_netlink_cmd_rcv_bufs_force);
 	conf_write(fp, " lvs_netlink_monitor_rcv_bufs = %u", global_data->lvs_netlink_monitor_rcv_bufs);
-	conf_write(fp, " lvs_netlink_monitor_rcv_bufs_force = %u", global_data->lvs_netlink_monitor_rcv_bufs_force);
-	conf_write(fp, " rs_init_notifies = %u", global_data->rs_init_notifies);
-	conf_write(fp, " no_checker_emails = %u", global_data->no_checker_emails);
+	conf_write(fp, " lvs_netlink_monitor_rcv_bufs_force = %d", global_data->lvs_netlink_monitor_rcv_bufs_force);
+	conf_write(fp, " rs_init_notifies = %d", global_data->rs_init_notifies);
+	conf_write(fp, " no_checker_emails = %d", global_data->no_checker_emails);
 #endif
 #ifdef _WITH_VRRP_
 	buf[0] = '\0';
@@ -567,10 +635,14 @@ dump_global_data(FILE *fp, data_t * data)
 	else if (global_data->vrrp_rx_bufs_policy & RX_BUFS_POLICY_ADVERT)
 		strcpy(buf, " rx_bufs_policy = ADVERT");
 	else if (global_data->vrrp_rx_bufs_policy & RX_BUFS_SIZE)
-		sprintf(buf, " rx_bufs_size = %lu", global_data->vrrp_rx_bufs_size);
+		sprintf(buf, " rx_bufs_size = %zu", global_data->vrrp_rx_bufs_size);
 	if (buf[0])
 		conf_write(fp, "%s", buf);
-	conf_write(fp, " rx_bufs_multiples = %u", global_data->vrrp_rx_bufs_multiples);
-	conf_write(fp, " umask = 0%o", global_data->umask);
+	conf_write(fp, " rx_bufs_multiples = %d", global_data->vrrp_rx_bufs_multiples);
+	conf_write(fp, " umask = 0%o", umask_val);
+	if (global_data->vrrp_startup_delay)
+		conf_write(fp, " vrrp_startup_delay = %g", global_data->vrrp_startup_delay / TIMER_HZ_DOUBLE);
+	if (global_data->log_unknown_vrids)
+		conf_write(fp, " log_unknown_vrids");
 #endif
 }
