@@ -28,6 +28,7 @@
 #include <stdbool.h>
 #ifdef HAVE_SIGNALFD
 #include <sys/signalfd.h>
+#include <sys/epoll.h>
 #endif
 #include <errno.h>
 #include <signal.h>
@@ -61,6 +62,9 @@
 #include "vrrp_daemon.h"
 #include "vrrp_parser.h"
 #include "vrrp_if.h"
+#ifdef _WITH_CN_PROC_
+#include "track_process.h"
+#endif
 #ifdef _WITH_JSON_
 #include "vrrp_json.h"
 #endif
@@ -92,6 +96,10 @@
 #ifdef _TSM_DEBUG_
 #include "vrrp_scheduler.h"
 #endif
+#if defined _PARSER_DEBUG_ || defined _DUMP_KEYWORDS_
+#include "parser.h"
+#endif
+#include "warnings.h"
 
 /* musl libc doesn't define the following */
 #ifndef	W_EXITCODE
@@ -101,39 +109,36 @@
 #define	WCOREFLAG		((int32_t)WCOREDUMP(0xffffffff))
 #endif
 
-#define	VERSION_STRING		PACKAGE_NAME " v" PACKAGE_VERSION " (" GIT_DATE ")"
-#define COPYRIGHT_STRING	"Copyright(C) 2001-" GIT_YEAR " Alexandre Cassen, <acassen@gmail.com>"
-
 #define CHILD_WAIT_SECS	5
 
 /* global var */
 const char *version_string = VERSION_STRING;		/* keepalived version */
-char *conf_file = KEEPALIVED_CONFIG_FILE;		/* Configuration file */
+const char *conf_file = KEEPALIVED_CONFIG_FILE;		/* Configuration file */
 int log_facility = LOG_DAEMON;				/* Optional logging facilities */
 bool reload;						/* Set during a reload */
-char *main_pidfile;					/* overrule default pidfile */
+const char *main_pidfile;				/* overrule default pidfile */
 static bool free_main_pidfile;
 #ifdef _WITH_LVS_
 pid_t checkers_child;					/* Healthcheckers child process ID */
-char *checkers_pidfile;					/* overrule default pidfile */
+const char *checkers_pidfile;				/* overrule default pidfile */
 static bool free_checkers_pidfile;
 #endif
 #ifdef _WITH_VRRP_
 pid_t vrrp_child;					/* VRRP child process ID */
-char *vrrp_pidfile;					/* overrule default pidfile */
+const char *vrrp_pidfile;				/* overrule default pidfile */
 static bool free_vrrp_pidfile;
 #endif
 #ifdef _WITH_BFD_
 pid_t bfd_child;					/* BFD child process ID */
-char *bfd_pidfile;					/* overrule default pidfile */
+const char *bfd_pidfile;				/* overrule default pidfile */
 static bool free_bfd_pidfile;
 #endif
 unsigned long daemon_mode;				/* VRRP/CHECK/BFD subsystem selection */
 #ifdef _WITH_SNMP_
-bool snmp;						/* Enable SNMP support */
+bool snmp_option;					/* Enable SNMP support */
 const char *snmp_socket;				/* Socket to use for SNMP agent */
 #endif
-static char *syslog_ident;				/* syslog ident if not default */
+static const char *syslog_ident;			/* syslog ident if not default */
 bool use_pid_dir;					/* Put pid files in /var/run/keepalived or @localstatedir@/run/keepalived */
 
 unsigned os_major;					/* Kernel version */
@@ -158,7 +163,6 @@ static struct {
 
 /* umask settings */
 bool umask_cmdline;
-static mode_t umask_val = S_IXUSR | S_IWGRP | S_IXGRP | S_IWOTH | S_IXOTH;
 
 /* Control producing core dumps */
 static bool set_core_dump_pattern = false;
@@ -167,7 +171,7 @@ static const char *core_dump_pattern = "core";
 static char *orig_core_dump_pattern = NULL;
 
 /* debug flags */
-#if defined _TIMER_CHECK_ || defined _SMTP_ALERT_DEBUG_ || defined _EPOLL_DEBUG_ || defined _EPOLL_THREAD_DUMP_ || defined _REGEX_DEBUG_ || defined _WITH_REGEX_TIMERS_ || defined _TSM_DEBUG_ || defined _VRRP_FD_DEBUG_ || defined _NETLINK_TIMERS_
+#if defined _TIMER_CHECK_ || defined _SMTP_ALERT_DEBUG_ || defined _EPOLL_DEBUG_ || defined _EPOLL_THREAD_DUMP_ || defined _REGEX_DEBUG_ || defined _WITH_REGEX_TIMERS_ || defined _TSM_DEBUG_ || defined _VRRP_FD_DEBUG_ || defined _NETLINK_TIMERS_ || defined _NETWORK_TIMESTAMP_ || defined _TRACK_PROCESS_DEBUG_ || defined _PARSER_DEBUG_ || defined _DUMP_KEYWORDS_
 #define WITH_DEBUG_OPTIONS 1
 #endif
 
@@ -198,6 +202,18 @@ static char vrrp_fd_debug;
 #ifdef _NETLINK_TIMERS_
 static char netlink_timer_debug;
 #endif
+#ifdef _NETWORK_TIMESTAMP_
+static char network_timestamp_debug;
+#endif
+#ifdef _TRACK_PROCESS_DEBUG_
+static char track_process_debug;
+#endif
+#ifdef _PARSER_DEBUG_
+static char parser_debug;
+#endif
+#ifdef _DUMP_KEYWORDS_
+static char dump_keywords;
+#endif
 
 void
 free_parent_mallocs_startup(bool am_child)
@@ -207,9 +223,9 @@ free_parent_mallocs_startup(bool am_child)
 		free_dirname();
 #endif
 #ifndef _MEM_CHECK_LOG_
-		FREE_PTR(syslog_ident);
+		FREE_CONST_PTR(syslog_ident);
 #else
-		free(syslog_ident);
+		free(no_const_char_p(syslog_ident));
 #endif
 		syslog_ident = NULL;
 
@@ -217,7 +233,7 @@ free_parent_mallocs_startup(bool am_child)
 	}
 
 	if (free_main_pidfile) {
-		FREE_PTR(main_pidfile);
+		FREE_CONST_PTR(main_pidfile);
 		free_main_pidfile = false;
 	}
 }
@@ -227,21 +243,21 @@ free_parent_mallocs_exit(void)
 {
 #ifdef _WITH_VRRP_
 	if (free_vrrp_pidfile)
-		FREE_PTR(vrrp_pidfile);
+		FREE_CONST_PTR(vrrp_pidfile);
 #endif
 #ifdef _WITH_LVS_
 	if (free_checkers_pidfile)
-		FREE_PTR(checkers_pidfile);
+		FREE_CONST_PTR(checkers_pidfile);
 #endif
 #ifdef _WITH_BFD_
 	if (free_bfd_pidfile)
-		FREE_PTR(bfd_pidfile);
+		FREE_CONST_PTR(bfd_pidfile);
 #endif
 
-	FREE_PTR(config_id);
+	FREE_CONST_PTR(config_id);
 }
 
-char *
+const char *
 make_syslog_ident(const char* name)
 {
 	size_t ident_len = strlen(name) + 1;
@@ -310,7 +326,7 @@ make_pidfile_name(const char* start, const char* instance, const char* extn)
 }
 
 #ifdef _WITH_VRRP_
-bool
+bool __attribute__ ((pure))
 running_vrrp(void)
 {
 	return (__test_bit(DAEMON_VRRP, &daemon_mode) &&
@@ -320,7 +336,7 @@ running_vrrp(void)
 #endif
 
 #ifdef _WITH_LVS_
-bool
+bool __attribute__ ((pure))
 running_checker(void)
 {
 	return (__test_bit(DAEMON_CHECKERS, &daemon_mode) &&
@@ -330,7 +346,7 @@ running_checker(void)
 #endif
 
 #ifdef _WITH_BFD_
-bool
+static bool __attribute__ ((pure))
 running_bfd(void)
 {
 	return (__test_bit(DAEMON_BFD, &daemon_mode) &&
@@ -358,7 +374,7 @@ find_keepalived_child_name(pid_t pid)
 	return NULL;
 }
 
-static vector_t *
+static const vector_t *
 global_init_keywords(void)
 {
 	/* global definitions mapping */
@@ -509,15 +525,25 @@ static bool reload_config(void)
 
 	read_config_file();
 
-	init_global_data(global_data, old_global_data);
+	init_global_data(global_data, old_global_data, false);
+
+	/* Update process name if necessary */
+	if (!global_data->process_name != !old_global_data->process_name ||
+	    (global_data->process_name && strcmp(global_data->process_name, old_global_data->process_name)))
+		set_process_name(global_data->process_name);
 
 #if HAVE_DECL_CLONE_NEWNET
+	if (override_namespace) {
+		FREE_CONST_PTR(global_data->network_namespace);
+		global_data->network_namespace = STRDUP(override_namespace);
+	}
+
 	if (!!old_global_data->network_namespace != !!global_data->network_namespace ||
 	    (global_data->network_namespace && strcmp(old_global_data->network_namespace, global_data->network_namespace))) {
 		log_message(LOG_INFO, "Cannot change network namespace at a reload - please restart %s", PACKAGE);
 		unsupported_change = true;
 	}
-	FREE_PTR(global_data->network_namespace);
+	FREE_CONST_PTR(global_data->network_namespace);
 	global_data->network_namespace = old_global_data->network_namespace;
 	old_global_data->network_namespace = NULL;
 #endif
@@ -527,10 +553,21 @@ static bool reload_config(void)
 		log_message(LOG_INFO, "Cannot change instance name at a reload - please restart %s", PACKAGE);
 		unsupported_change = true;
 	}
-	FREE_PTR(global_data->instance_name);
+	FREE_CONST_PTR(global_data->instance_name);
 	global_data->instance_name = old_global_data->instance_name;
 	old_global_data->instance_name = NULL;
 
+#ifdef _WITH_NFTABLES_
+	if (!!old_global_data->vrrp_nf_table_name != !!global_data->vrrp_nf_table_name ||
+	    (global_data->vrrp_nf_table_name && strcmp(old_global_data->vrrp_nf_table_name, global_data->vrrp_nf_table_name))) {
+		log_message(LOG_INFO, "Cannot change nftables table name at a reload - please restart %s", PACKAGE);
+		unsupported_change = true;
+	}
+	FREE_CONST_PTR(global_data->vrrp_nf_table_name);
+	global_data->vrrp_nf_table_name = old_global_data->vrrp_nf_table_name;
+	old_global_data->vrrp_nf_table_name = NULL;
+#endif
+
 	if (unsupported_change) {
 		/* We cannot reload the configuration, so continue with the old config */
 		free_global_data (global_data);
@@ -539,6 +576,7 @@ static bool reload_config(void)
 	else
 		free_global_data (old_global_data);
 
+
 	return !unsupported_change;
 }
 
@@ -559,7 +597,7 @@ propagate_signal(__attribute__((unused)) void *v, int sig)
 		start_vrrp_child();
 #endif
 #ifdef _WITH_LVS_
-	if (sig == SIGHUP) {
+	if (sig == SIGHUP || sig == SIGUSR1) {
 		if (checkers_child > 0)
 			kill(checkers_child, sig);
 		else if (running_checker())
@@ -585,14 +623,13 @@ sigend(__attribute__((unused)) void *v, __attribute__((unused)) int sig)
 	int wait_count = 0;
 	struct timeval start_time, now;
 #ifdef HAVE_SIGNALFD
-	struct timeval timeout = {
-		.tv_sec = child_wait_time,
-		.tv_usec = 0
-	};
+	int timeout = child_wait_time * 1000;
 	int signal_fd = master->signal_fd;
-	fd_set read_set;
 	struct signalfd_siginfo siginfo;
 	sigset_t sigmask;
+	struct epoll_event ev = { .events = EPOLLIN, .data.fd = master->signal_fd };
+	int efd;
+	int wstatus;
 #else
 	sigset_t old_set, child_wait;
 	struct timespec timeout = {
@@ -611,7 +648,6 @@ sigend(__attribute__((unused)) void *v, __attribute__((unused)) int sig)
 	sigemptyset(&sigmask);
 	sigaddset(&sigmask, SIGCHLD);
 	signalfd(signal_fd, &sigmask, 0);
-	FD_ZERO(&read_set);
 #else
 	sigmask_func(0, NULL, &old_set);
 	if (!sigismember(&old_set, SIGCHLD)) {
@@ -653,23 +689,27 @@ sigend(__attribute__((unused)) void *v, __attribute__((unused)) int sig)
 	}
 #endif
 
+#ifdef HAVE_SIGNALFD
+	efd = epoll_create(1);
+	epoll_ctl(efd, EPOLL_CTL_ADD, signal_fd, &ev);
+#endif
+
 	gettimeofday(&start_time, NULL);
 	while (wait_count) {
 #ifdef HAVE_SIGNALFD
-		FD_SET(signal_fd, &read_set);
-		ret = select(signal_fd + 1, &read_set, NULL, NULL, &timeout);
+		ret = epoll_wait(efd, &ev, 1, timeout);
 		if (ret == 0)
 			break;
 		if (ret == -1) {
-			if (errno == EINTR)
+			if (check_EINTR(errno))
 				continue;
 
-			log_message(LOG_INFO, "Terminating select returned errno %d", errno);
+			log_message(LOG_INFO, "Terminating epoll_wait returned errno %d", errno);
 			break;
 		}
 
-		if (!FD_ISSET(signal_fd, &read_set)) {
-			log_message(LOG_INFO, "Terminating select did not return select_fd");
+		if (ev.data.fd != signal_fd) {
+			log_message(LOG_INFO, "Terminating epoll_wait did not return signal_fd");
 			continue;
 		}
 
@@ -685,6 +725,17 @@ sigend(__attribute__((unused)) void *v, __attribute__((unused)) int sig)
 #ifdef _WITH_VRRP_
 		if (vrrp_child > 0 && vrrp_child == (pid_t)siginfo.ssi_pid) {
 			report_child_status(status, vrrp_child, PROG_VRRP);
+			ret = waitpid(vrrp_child, &wstatus, WNOHANG);
+			if (ret == 0)
+				continue;
+			if (ret == -1) {
+				if (check_EINTR(errno))
+					continue;
+				log_message(LOG_INFO, "Wait for vrrp child return errno %d", errno);
+			}
+
+			/* We could check ret == vrrp_child, but it seems unneccessary */
+
 			vrrp_child = 0;
 			wait_count--;
 		}
@@ -693,6 +744,14 @@ sigend(__attribute__((unused)) void *v, __attribute__((unused)) int sig)
 #ifdef _WITH_LVS_
 		if (checkers_child > 0 && checkers_child == (pid_t)siginfo.ssi_pid) {
 			report_child_status(status, checkers_child, PROG_CHECK);
+			ret = waitpid(checkers_child, &wstatus, WNOHANG);
+			if (ret == 0)
+				continue;
+			if (ret == -1) {
+				if (check_EINTR(errno))
+					continue;
+				log_message(LOG_INFO, "Wait for checker child return errno %d", errno);
+			}
 			checkers_child = 0;
 			wait_count--;
 		}
@@ -700,6 +759,14 @@ sigend(__attribute__((unused)) void *v, __attribute__((unused)) int sig)
 #ifdef _WITH_BFD_
 		if (bfd_child > 0 && bfd_child == (pid_t)siginfo.ssi_pid) {
 			report_child_status(status, bfd_child, PROG_BFD);
+			ret = waitpid(bfd_child, &wstatus, WNOHANG);
+			if (ret == 0)
+				continue;
+			if (ret == -1) {
+				if (check_EINTR(errno))
+					continue;
+				log_message(LOG_INFO, "Wait for bfd child return errno %d", errno);
+			}
 			bfd_child = 0;
 			wait_count--;
 		}
@@ -708,9 +775,9 @@ sigend(__attribute__((unused)) void *v, __attribute__((unused)) int sig)
 #else
 		ret = sigtimedwait(&child_wait, NULL, &timeout);
 		if (ret == -1) {
-			if (errno == EINTR)
+			if (check_EINTR(errno))
 				continue;
-			if (errno == EAGAIN)
+			if (check_EAGAIN(errno))
 				break;
 		}
 
@@ -736,29 +803,29 @@ sigend(__attribute__((unused)) void *v, __attribute__((unused)) int sig)
 			wait_count--;
 		}
 #endif
-
 #endif
 
 		if (wait_count) {
 			gettimeofday(&now, NULL);
-			timeout.tv_sec = child_wait_time - (now.tv_sec - start_time.tv_sec);
 #ifdef HAVE_SIGNALFD
-			timeout.tv_usec = (start_time.tv_usec - now.tv_usec);
-			if (timeout.tv_usec < 0) {
-				timeout.tv_usec += 1000000L;
-				timeout.tv_sec--;
-			}
+			timeout = (child_wait_time - (now.tv_sec - start_time.tv_sec)) * 1000 + (start_time.tv_usec - now.tv_usec) / 1000;
+			if (timeout < 0)
+				break;
 #else
+			timeout.tv_sec = child_wait_time - (now.tv_sec - start_time.tv_sec);
 			timeout.tv_nsec = (start_time.tv_usec - now.tv_usec) * 1000;
 			if (timeout.tv_nsec < 0) {
 				timeout.tv_nsec += 1000000000L;
 				timeout.tv_sec--;
 			}
-#endif
 			if (timeout.tv_sec < 0)
 				break;
+#endif
 		}
 	}
+#ifdef HAVE_SIGNALFD
+	close(efd);
+#endif
 
 	/* A child may not have terminated, so force its termination */
 #ifdef _WITH_VRRP_
@@ -875,24 +942,32 @@ static mode_t
 set_umask(const char *optarg)
 {
 	long umask_long;
-	mode_t umask_val;
+	mode_t umask_bits;
 	char *endptr;
 
 	umask_long = strtoll(optarg, &endptr, 0);
 
-	if (*endptr || umask_long < 0 || umask_long & ~0777L) {
+	if (*endptr || umask_long < 0 || umask_long & ~(S_IRWXU | S_IRWXG | S_IRWXO)) {
 		fprintf(stderr, "Invalid --umask option %s", optarg);
 		return 0;
 	}
 
-	umask_val = umask_long & 0777;
-	umask(umask_val);
+	umask_bits = umask_long & (S_IRWXU | S_IRWXG | S_IRWXO);
+	umask(umask_bits);
 
 	umask_cmdline = true;
 
-	return umask_val;
+#ifdef _MEM_CHECK_
+	update_mem_check_log_perms(umask_bits);
+#endif
+#ifdef ENABLE_LOG_TO_FILE
+	update_log_file_perms(umask_bits);
+#endif
+
+	return umask_bits;
 }
 
+RELAX_SUGGEST_ATTRIBUTE_CONST_START
 void
 initialise_debug_options(void)
 {
@@ -901,15 +976,15 @@ initialise_debug_options(void)
 
 	if (prog_type == PROG_TYPE_PARENT)
 		mask = 1 << PROG_TYPE_PARENT;
-#if _WITH_BFD_
+#ifdef _WITH_BFD_
 	else if (prog_type == PROG_TYPE_BFD)
 		mask = 1 << PROG_TYPE_BFD;
 #endif
-#if _WITH_LVS_
+#ifdef _WITH_LVS_
 	else if (prog_type == PROG_TYPE_CHECKER)
 		mask = 1 << PROG_TYPE_CHECKER;
 #endif
-#if _WITH_VRRP_
+#ifdef _WITH_VRRP_
 	else if (prog_type == PROG_TYPE_VRRP)
 		mask = 1 << PROG_TYPE_VRRP;
 #endif
@@ -941,8 +1016,23 @@ initialise_debug_options(void)
 #ifdef _NETLINK_TIMERS_
 	do_netlink_timers = !!(netlink_timer_debug & mask);
 #endif
+#ifdef _NETWORK_TIMESTAMP_
+	do_network_timestamp = !!(network_timestamp_debug & mask);
+#endif
+#ifdef _WITH_CN_PROC_
+#ifdef _TRACK_PROCESS_DEBUG_
+	do_track_process_debug = !!(track_process_debug & mask);
+#endif
+#endif
+#ifdef _PARSER_DEBUG_
+	do_parser_debug = !!(parser_debug & mask);
+#endif
+#ifdef _DUMP_KEYWORDS_
+	do_dump_keywords = !!(dump_keywords & mask);
+#endif
 #endif
 }
+RELAX_SUGGEST_ATTRIBUTE_CONST_END
 
 #ifdef  WITH_DEBUG_OPTIONS
 static void
@@ -956,13 +1046,13 @@ set_debug_options(const char *options)
 	all_processes = 1;
 #else
 	all_processes = (1 << PROG_TYPE_PARENT);
-#if _WITH_BFD_
+#ifdef _WITH_BFD_
 	all_processes |= (1 << PROG_TYPE_BFD);
 #endif
-#if _WITH_LVS_
+#ifdef _WITH_LVS_
 	all_processes |= (1 << PROG_TYPE_CHECKER);
 #endif
-#if _WITH_VRRP_
+#ifdef _WITH_VRRP_
 	all_processes |= (1 << PROG_TYPE_VRRP);
 #endif
 #endif
@@ -995,12 +1085,24 @@ set_debug_options(const char *options)
 #ifdef _NETLINK_TIMERS_
 		netlink_timer_debug = all_processes;
 #endif
+#ifdef _NETWORK_TIMESTAMP_
+		network_timestamp_debug = all_processes;
+#endif
+#ifdef _TRACK_PROCESS_DEBUG_
+		track_process_debug = all_processes;
+#endif
+#ifdef _PARSER_DEBUG_
+		parser_debug = all_processes;
+#endif
+#ifdef _DUMP_KEYWORDS_
+		dump_keywords = all_processes;
+#endif
 
 		return;
 	}
 
 	opt_p = options;
-	do {
+	while (*opt_p) {
 		if (!isupper(*opt_p)) {
 			fprintf(stderr, "Unknown debug option'%c' in '%s'\n", *opt_p, options);
 			return;
@@ -1019,17 +1121,17 @@ set_debug_options(const char *options)
 				case 'p':
 					processes |= (1 << PROG_TYPE_PARENT);
 					break;
-#if _WITH_BFD_
+#ifdef _WITH_BFD_
 				case 'b':
 					processes |= (1 << PROG_TYPE_BFD);
 					break;
 #endif
-#if _WITH_LVS_
+#ifdef _WITH_LVS_
 				case 'c':
 					processes |= (1 << PROG_TYPE_CHECKER);
 					break;
 #endif
-#if _WITH_VRRP_
+#ifdef _WITH_VRRP_
 				case 'v':
 					processes |= (1 << PROG_TYPE_VRRP);
 					break;
@@ -1088,12 +1190,32 @@ set_debug_options(const char *options)
 		case 'N':
 			netlink_timer_debug = processes;
 			break;
+#endif
+#ifdef _NETWORK_TIMESTAMP_
+		case 'P':
+			network_timestamp_debug = processes;
+			break;
+#endif
+#ifdef _TRACK_PROCESS_DEBUG_
+		case 'O':
+			track_process_debug = processes;
+			break;
+#endif
+#ifdef _PARSER_DEBUG_
+		case 'C':
+			parser_debug = processes;
+			break;
+#endif
+#ifdef _DUMP_KEYWORDS_
+		case 'K':
+			dump_keywords = processes;
+			break;
 #endif
 		default:
 			fprintf(stderr, "Unknown debug type '%c' in '%s'\n", opt, options);
 			return;
 		}
-	} while (opt_p && *opt_p);
+	}
 }
 #endif
 
@@ -1180,7 +1302,7 @@ usage(const char *prog)
 #ifdef _EPOLL_THREAD_DUMP_
 	fprintf(stderr, "                                   D - epoll thread dump debug\n");
 #endif
-#ifdef _VRRP_FD_DEBUG
+#ifdef _VRRP_FD_DEBUG_
 	fprintf(stderr, "                                   F - vrrp fd dump debug\n");
 #endif
 #ifdef _REGEX_DEBUG_
@@ -1194,6 +1316,18 @@ usage(const char *prog)
 #endif
 #ifdef _NETLINK_TIMERS_
 	fprintf(stderr, "                                   N - netlink timer debug\n");
+#endif
+#ifdef _NETWORK_TIMESTAMP_
+	fprintf(stderr, "                                   P - network timestamp debug\n");
+#endif
+#ifdef _TRACK_PROCESS_DEBUG_
+	fprintf(stderr, "                                   O - track process debug\n");
+#endif
+#ifdef _PARSER_DEBUG_
+	fprintf(stderr, "                                   C - parser (config) debug\n");
+#endif
+#ifdef _DUMP_KEYWORDS_
+	fprintf(stderr, "                                   K - dump keywords\n");
 #endif
 	fprintf(stderr, "                                 Example --debug=TpMEvcp\n");
 #endif
@@ -1312,10 +1446,8 @@ parse_cmdline(int argc, char **argv)
 
 		/* Check for an empty option argument. For example --use-file= returns
 		 * a 0 length option, which we don't want */
-		if (longindex >= 0 && long_options[longindex].has_arg == required_argument && optarg && !optarg[0]) {
+		if (longindex >= 0 && long_options[longindex].has_arg == required_argument && optarg && !optarg[0])
 			c = ':';
-			optarg = NULL;
-		}
 
 		switch (c) {
 		case 'v':
@@ -1461,7 +1593,7 @@ parse_cmdline(int argc, char **argv)
 #endif
 #ifdef _WITH_SNMP_
 		case 'x':
-			snmp = 1;
+			snmp_option = true;
 			break;
 		case 'A':
 			snmp_socket = optarg;
@@ -1482,14 +1614,12 @@ parse_cmdline(int argc, char **argv)
 #endif
 #if HAVE_DECL_CLONE_NEWNET
 		case 's':
-			override_namespace = MALLOC(strlen(optarg) + 1);
-			strcpy(override_namespace, optarg);
+			override_namespace = optarg;
 			break;
 #endif
 		case 'i':
-			FREE_PTR(config_id);
-			config_id = MALLOC(strlen(optarg) + 1);
-			strcpy(config_id, optarg);
+			FREE_CONST_PTR(config_id);
+			config_id = STRDUP(optarg);
 			break;
 		case 4:			/* --signum */
 			signum = get_signum(optarg);
@@ -1630,7 +1760,7 @@ keepalived_main(int argc, char **argv)
 #endif
 
 	/* Set default file creation mask */
-	umask(022);
+	umask(umask_val);
 
 	/* Open log with default settings so we can log initially */
 	openlog(PACKAGE_NAME, LOG_PID, log_facility);
@@ -1663,9 +1793,7 @@ keepalived_main(int argc, char **argv)
 		/* config_id defaults to hostname */
 		if (!config_id) {
 			end = strchrnul(uname_buf.nodename, '.');
-			config_id = MALLOC((size_t)(end - uname_buf.nodename) + 1);
-			strncpy(config_id, uname_buf.nodename, (size_t)(end - uname_buf.nodename));
-			config_id[end - uname_buf.nodename] = '\0';
+			config_id = STRNDUP(uname_buf.nodename, (size_t)(end - uname_buf.nodename));
 		}
 	}
 
@@ -1729,20 +1857,22 @@ keepalived_main(int argc, char **argv)
 	}
 
 	global_data = alloc_global_data();
-	global_data->umask = umask_val;
 
 	read_config_file();
 
-	init_global_data(global_data, NULL);
+	init_global_data(global_data, NULL, false);
+
+	/* Update process name if necessary */
+	if (global_data->process_name)
+		set_process_name(global_data->process_name);
 
 #if HAVE_DECL_CLONE_NEWNET
 	if (override_namespace) {
 		if (global_data->network_namespace) {
 			log_message(LOG_INFO, "Overriding config net_namespace '%s' with command line namespace '%s'", global_data->network_namespace, override_namespace);
-			FREE(global_data->network_namespace);
+			FREE_CONST(global_data->network_namespace);
 		}
-		global_data->network_namespace = override_namespace;
-		override_namespace = NULL;
+		global_data->network_namespace = STRDUP(override_namespace);
 	}
 #endif
 
@@ -1784,6 +1914,12 @@ keepalived_main(int argc, char **argv)
 		}
 	}
 
+	/* If we want to monitor processes, we have to do it before calling
+	 * setns() */
+#ifdef _WITH_CN_PROC_
+	open_track_processes();
+#endif
+
 #if HAVE_DECL_CLONE_NEWNET
 	if (global_data->network_namespace) {
 		if (global_data->network_namespace && !set_namespaces(global_data->network_namespace)) {
@@ -1857,7 +1993,7 @@ keepalived_main(int argc, char **argv)
 	if (!__test_bit(DONT_FORK_BIT, &debug) &&
 	    xdaemon(false, false, true) > 0) {
 		closelog();
-		FREE_PTR(config_id);
+		FREE_CONST_PTR(config_id);
 		FREE_PTR(orig_core_dump_pattern);
 		close_std_fd();
 		exit(0);
@@ -1935,12 +2071,12 @@ end:
 	closelog();
 
 #ifndef _MEM_CHECK_LOG_
-	FREE_PTR(syslog_ident);
+	FREE_CONST_PTR(syslog_ident);
 #else
 	if (syslog_ident)
-		free(syslog_ident);
+		free(no_const_char_p(syslog_ident));
 #endif
 	close_std_fd();
 
-	exit(exit_code);
+	return exit_code;
 }
