@@ -30,6 +30,10 @@
 #include <stdio.h>
 #include <sys/socket.h>
 
+#ifdef _HAVE_SCHED_RT_
+#include <sched.h>
+#endif
+
 #ifdef HAVE_LINUX_NETFILTER_X_TABLES_H
 #include <linux/netfilter/x_tables.h>
 #endif
@@ -38,6 +42,10 @@
 #include <linux/netfilter/ipset/ip_set.h>
 #endif
 
+#ifdef _WITH_NFTABLES_
+#include <linux/netfilter/nf_tables.h>
+#endif
+
 #if HAVE_DECL_RLIMIT_RTTIME == 1
 #include <sys/resource.h>
 #endif
@@ -72,19 +80,39 @@ typedef struct _email {
 	char				*addr;
 } email_t;
 
+#ifdef _WITH_LVS_
+typedef enum {
+	LVS_NO_FLUSH,
+	LVS_FLUSH_FULL,
+	LVS_FLUSH_VS
+} lvs_flush_t;
+#endif
+
 /* Configuration data root */
 typedef struct _data {
+	const char 			*process_name;
+#ifdef _WITH_VRRP_
+	const char			*vrrp_process_name;
+#endif
+#ifdef _WITH_LVS_
+	const char			*lvs_process_name;
+#endif
+#ifdef _WITH_BFD_
+	const char			*bfd_process_name;
+#endif
 #if HAVE_DECL_CLONE_NEWNET
-	char				*network_namespace;	/* network namespace name */
+	const char			*network_namespace;	/* network namespace name */
 	bool				namespace_with_ipsets;	/* override for namespaces with ipsets on Linux < 3.13 */
 #endif
-	char				*local_name;
-	char				*instance_name;		/* keepalived instance name */
+	const char			*local_name;
+	const char			*instance_name;		/* keepalived instance name */
+#ifdef _WITH_LINKBEAT_
 	bool				linkbeat_use_polling;
-	char				*router_id;
-	char				*email_from;
+#endif
+	const char			*router_id;
+	const char			*email_from;
 	struct sockaddr_storage		smtp_server;
-	char				*smtp_helo_name;
+	const char			*smtp_helo_name;
 	unsigned long			smtp_connection_to;
 	list				email;
 	int				smtp_alert;
@@ -93,7 +121,7 @@ typedef struct _data {
 	bool				allow_if_changes;
 	bool				no_email_faults;
 	int				smtp_alert_vrrp;
-	char				*default_ifname;	/* Name of default interface */
+	const char			*default_ifname;	/* Name of default interface */
 	interface_t			*default_ifp;		/* Default interface for static addresses */
 #endif
 #ifdef _WITH_LVS_
@@ -101,10 +129,12 @@ typedef struct _data {
 	int				lvs_tcpfin_timeout;
 	int				lvs_udp_timeout;
 	int				smtp_alert_checker;
+	bool				checker_log_all_failures;
 #ifdef _WITH_VRRP_
 	struct lvs_syncd_config		lvs_syncd;
 #endif
 	bool				lvs_flush;		/* flush any residual LVS config at startup */
+	lvs_flush_t			lvs_flush_onstop;	/* flush any LVS config at shutdown */
 #endif
 #ifdef _WITH_VRRP_
 	struct sockaddr_in		vrrp_mcast_group4;
@@ -119,7 +149,8 @@ typedef struct _data {
 	unsigned			vrrp_gna_interval;
 	bool				vrrp_lower_prio_no_advert;
 	bool				vrrp_higher_prio_send_advert;
-	int				vrrp_version;	/* VRRP version (2 or 3) */
+	int				vrrp_version;		/* VRRP version (2 or 3) */
+#ifdef _WITH_IPTABLES_
 	char				vrrp_iptables_inchain[XT_EXTENSION_MAXNAMELEN];
 	char				vrrp_iptables_outchain[XT_EXTENSION_MAXNAMELEN];
 #ifdef _HAVE_LIBIPSET_
@@ -127,6 +158,13 @@ typedef struct _data {
 	char				vrrp_ipset_address[IPSET_MAXNAMELEN];
 	char				vrrp_ipset_address6[IPSET_MAXNAMELEN];
 	char				vrrp_ipset_address_iface6[IPSET_MAXNAMELEN];
+#endif
+#endif
+#ifdef _WITH_NFTABLES_
+	const char			*vrrp_nf_table_name;
+	int				vrrp_nf_chain_priority;
+	bool				vrrp_nf_counters;
+	bool				vrrp_nf_ifindex;
 #endif
 	bool				vrrp_check_unicast_src;
 	bool				vrrp_skip_check_adv_addr;
@@ -136,6 +174,7 @@ typedef struct _data {
 	bool				vrrp_no_swap;
 #ifdef _HAVE_SCHED_RT_
 	unsigned			vrrp_realtime_priority;
+	cpu_set_t			vrrp_cpu_mask;
 #if HAVE_DECL_RLIMIT_RTTIME == 1
 	rlim_t				vrrp_rlimit_rt;
 #endif
@@ -147,6 +186,7 @@ typedef struct _data {
 	bool				checker_no_swap;
 #ifdef _HAVE_SCHED_RT_
 	unsigned			checker_realtime_priority;
+	cpu_set_t			checker_cpu_mask;
 #if HAVE_DECL_RLIMIT_RTTIME == 1
 	rlim_t				checker_rlimit_rt;
 #endif
@@ -158,6 +198,7 @@ typedef struct _data {
 	bool				bfd_no_swap;
 #ifdef _HAVE_SCHED_RT_
 	unsigned			bfd_realtime_priority;
+	cpu_set_t			bfd_cpu_mask;
 #if HAVE_DECL_RLIMIT_RTTIME == 1
 	rlim_t				bfd_rlimit_rt;
 #endif
@@ -170,9 +211,12 @@ typedef struct _data {
 #ifdef _WITH_LVS_
 	notify_fifo_t			lvs_notify_fifo;
 #endif
+#ifdef _WITH_VRRP_
+	int				vrrp_notify_priority_changes;
+#endif
 #ifdef _WITH_SNMP_
 	bool				enable_traps;
-	char				*snmp_socket;
+	const char			*snmp_socket;
 #ifdef _WITH_VRRP_
 #ifdef _WITH_SNMP_VRRP_
 	bool				enable_snmp_vrrp;
@@ -190,13 +234,17 @@ typedef struct _data {
 #endif
 #ifdef _WITH_DBUS_
 	bool				enable_dbus;
-	char				*dbus_service_name;
+	const char			*dbus_service_name;
 #endif
 #ifdef _WITH_VRRP_
 	unsigned			vrrp_netlink_cmd_rcv_bufs;
 	bool				vrrp_netlink_cmd_rcv_bufs_force;
 	unsigned			vrrp_netlink_monitor_rcv_bufs;
 	bool				vrrp_netlink_monitor_rcv_bufs_force;
+#ifdef _WITH_CN_PROC_
+	unsigned			process_monitor_rcv_bufs;
+	bool				process_monitor_rcv_bufs_force;
+#endif
 #endif
 #ifdef _WITH_LVS_
 	unsigned			lvs_netlink_cmd_rcv_bufs;
@@ -212,8 +260,9 @@ typedef struct _data {
 	int				vrrp_rx_bufs_policy;
 	size_t				vrrp_rx_bufs_size;
 	int				vrrp_rx_bufs_multiples;
+	unsigned			vrrp_startup_delay;
+	bool				log_unknown_vrids;
 #endif
-	mode_t				umask;			/* mask for file creation */
 } data_t;
 
 /* Global vars exported */
@@ -221,9 +270,9 @@ extern data_t *global_data;	/* Global configuration data */
 extern data_t *old_global_data;	/* Old global configuration data - used during reload */
 
 /* Prototypes */
-extern void alloc_email(char *);
+extern void alloc_email(const char *);
 extern data_t *alloc_global_data(void);
-extern void init_global_data(data_t *, data_t *);
+extern void init_global_data(data_t *, data_t *, bool);
 extern void free_global_data(data_t *);
 extern void dump_global_data(FILE *, data_t *);
 
