@@ -17,7 +17,7 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2016 Alexandre Cassen, <acassen@gmail.com>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #ifndef _VRRP_IPSET_H
@@ -26,10 +26,12 @@
 #define LIBIPSET_NFPROTO_H
 #include "vrrp_ipaddress.h"
 
+#define DEFAULT_IPSET_NAME	"keepalived"
+
 bool add_ipsets(bool);
 bool remove_ipsets(void);
 bool has_ipset_setname(void*, const char *);
-bool ipset_init(void);
+bool ipset_initialise(void);
 void* ipset_session_start(void);
 void ipset_session_end(void*);
 void ipset_entry(void*, int cmd, const ip_address_t*);
