@@ -34,7 +34,9 @@
 #include <sys/resource.h>
 
 #ifdef THREAD_DUMP
+#ifdef _WITH_SNMP_
 #include "snmp.h"
+#endif
 #include "scheduler.h"
 #include "smtp.h"
 #include "vrrp_track.h"
@@ -44,7 +46,6 @@
 #include "vrrp_arp.h"
 #include "vrrp_ndisc.h"
 #include "keepalived_netlink.h"
-#include "vrrp_iptables.h"
 #ifdef _HAVE_FIB_ROUTING_
 #include "vrrp_iprule.h"
 #include "vrrp_iproute.h"
@@ -60,7 +61,7 @@
 #include "memory.h"
 #include "bitops.h"
 #include "rttables.h"
-#if defined _WITH_SNMP_RFC || defined _WITH_SNMP_VRRP_
+#if defined _WITH_SNMP_RFC_ || defined _WITH_SNMP_VRRP_
   #include "vrrp_snmp.h"
 #endif
 #ifdef _WITH_DBUS_
@@ -78,17 +79,23 @@
 #ifdef _WITH_BFD_
 #include "bfd_daemon.h"
 #endif
+#ifdef _WITH_FIREWALL_
+#include "vrrp_firewall.h"
+#endif
+#ifdef _WITH_CN_PROC_
+#include "track_process.h"
+#endif
 
 /* Global variables */
 bool non_existent_interface_specified;
 
 /* Forward declarations */
 #ifndef _DEBUG_
-static int print_vrrp_data(thread_t * thread);
-static int print_vrrp_stats(thread_t * thread);
-static int reload_vrrp_thread(thread_t * thread);
+static int print_vrrp_data(thread_ref_t);
+static int print_vrrp_stats(thread_ref_t);
+static int reload_vrrp_thread(thread_ref_t);
 #ifdef _WITH_JSON_
-static int print_vrrp_json(thread_t * thread);
+static int print_vrrp_json(thread_ref_t);
 #endif
 #endif
 #ifdef _WITH_PERF_
@@ -96,7 +103,7 @@ perf_t perf_run = PERF_NONE;
 #endif
 
 /* local variables */
-static char *vrrp_syslog_ident;
+static const char *vrrp_syslog_ident;
 #ifndef _DEBUG_
 static bool two_phase_terminate;
 #endif
@@ -144,16 +151,9 @@ dump_vrrp_fd(void)
 static void
 set_vrrp_max_fds(void)
 {
-	struct rlimit orig_rlim, rlim;
-
 	if (!vrrp_data->vrrp)
 		return;
 
-	if (getrlimit(RLIMIT_NOFILE, &orig_rlim) == -1) {
-		log_message(LOG_INFO, "Failed to get RLIMIT_NOFILE - errno %d (%m)", errno);
-		return;
-	}
-
 	/* Allow:
 	 * 2 per vrrp instance - always needed for VMAC instances
 	 *
@@ -179,21 +179,7 @@ set_vrrp_max_fds(void)
 	 *
 	 * 20 spare (in case we have forgotten anything)
 	 */
-	rlim.rlim_cur = LIST_SIZE(vrrp_data->vrrp) * 2 + 21 + 20;
-	if (rlim.rlim_cur <= orig_rlim.rlim_cur)
-		return;
-
-	rlim.rlim_max = rlim.rlim_cur;
-
-	if (setrlimit(RLIMIT_NOFILE, &rlim) == -1) {
-		log_message(LOG_INFO, "Failed to set file number limit to %" PRI_rlim_t " - errno %d (%m)", rlim.rlim_cur, errno);
-		return;
-	}
-
-	log_message(LOG_INFO, "Set maximum open files to %" PRI_rlim_t, rlim.rlim_cur);
-
-	/* We don't want child processes to get excessive limits */
-	set_child_rlimit(RLIMIT_NOFILE, &orig_rlim);
+	set_max_file_limit(LIST_SIZE(vrrp_data->vrrp) * 2 + vrrp_data->num_smtp_alert + 21 + 20);
 }
 
 #ifdef _WITH_LVS_
@@ -222,9 +208,9 @@ vrrp_terminate_phase2(int exit_status)
 		report_and_clear_netlink_timers("Completed shutdown instances");
 #endif
 
-#if defined _WITH_SNMP_RFC || defined _WITH_SNMP_VRRP_
+#if defined _WITH_SNMP_RFC_ || defined _WITH_SNMP_VRRP_
 	if (
-#ifdef _WITH_SNMP_RFC_
+#ifdef _WITH_SNMP_VRRP_
 	    global_data->enable_snmp_vrrp ||
 #endif
 #ifdef _WITH_SNMP_RFCV2_
@@ -233,7 +219,7 @@ vrrp_terminate_phase2(int exit_status)
 #ifdef _WITH_SNMP_RFCV3_
 	    global_data->enable_snmp_rfcv3 ||
 #endif
-	    false)
+	    snmp_option)
 		vrrp_snmp_agent_close();
 #endif
 
@@ -249,6 +235,9 @@ vrrp_terminate_phase2(int exit_status)
 	master = NULL;
 	gratuitous_arp_close();
 	ndisc_close();
+#ifdef _WITH_LINKBEAT_
+	close_interface_linkbeat();
+#endif
 
 #ifdef _WITH_DBUS_
 	if (global_data->enable_dbus)
@@ -282,10 +271,10 @@ vrrp_terminate_phase2(int exit_status)
 	closelog();
 
 #ifndef _MEM_CHECK_LOG_
-	FREE_PTR(vrrp_syslog_ident);
+	FREE_CONST_PTR(vrrp_syslog_ident);
 #else
 	if (vrrp_syslog_ident)
-		free(vrrp_syslog_ident);
+		free(no_const_char_p(vrrp_syslog_ident));
 #endif
 	close_std_fd();
 
@@ -296,16 +285,16 @@ vrrp_terminate_phase2(int exit_status)
 }
 
 static int
-vrrp_shutdown_backstop_thread(thread_t *thread)
+vrrp_shutdown_backstop_thread(thread_ref_t thread)
 {
 	int count = 0;
-	thread_t *t;
+	thread_ref_t t;
 
 	/* Force terminate all script processes */
 	if (thread->master->child.rb_root.rb_node)
 		script_killall(thread->master, SIGKILL, true);
 
-	rb_for_each_entry_cached(t, &thread->master->child, n)
+	rb_for_each_entry_cached_const(t, &thread->master->child, n)
 		count++;
 
 	log_message(LOG_ERR, "Backstop thread invoked: shutdown timer %srunning, child count %d",
@@ -317,7 +306,7 @@ vrrp_shutdown_backstop_thread(thread_t *thread)
 }
 
 static int
-vrrp_shutdown_timer_thread(thread_t *thread)
+vrrp_shutdown_timer_thread(thread_ref_t thread)
 {
 	thread->master->shutdown_timer_running = false;
 
@@ -338,6 +327,11 @@ vrrp_terminate_phase1(bool schedule_next_thread)
 		run_perf("vrrp", global_data->network_namespace, global_data->instance_name);
 #endif
 
+#ifdef _WITH_CN_PROC_
+	/* Stop monitoring process terminations */
+	end_process_monitor();
+#endif
+
 	/* Terminate all script processes */
 	if (master->child.rb_root.rb_node)
 		script_killall(master, SIGTERM, true);
@@ -349,6 +343,15 @@ vrrp_terminate_phase1(bool schedule_next_thread)
 		report_and_clear_netlink_timers("Start shutdown");
 #endif
 
+#ifdef _WITH_LVS_
+        if (global_data->lvs_syncd.vrrp) {
+                /* Stop syncd if controlled by this VRRP instance. */
+                ipvs_syncd_cmd(IPVS_STOPDAEMON, &global_data->lvs_syncd,
+                               (global_data->lvs_syncd.vrrp->state == VRRP_STATE_MAST) ? IPVS_MASTER: IPVS_BACKUP,
+                               true, false);
+        }
+#endif
+
 	/* Ensure any interfaces are in backup mode,
 	 * sending a priority 0 vrrp message
 	 */
@@ -363,8 +366,8 @@ vrrp_terminate_phase1(bool schedule_next_thread)
 	if (vrrp_data->vrrp_track_files)
 		stop_track_files();
 
-#ifdef _HAVE_LIBIPTC_
-	iptables_fini();
+#ifdef _WITH_FIREWALL_
+	firewall_fini();
 #endif
 
 	/* Clear static entries */
@@ -405,7 +408,7 @@ vrrp_terminate_phase1(bool schedule_next_thread)
 
 #ifndef _DEBUG_
 static int
-start_vrrp_termination_thread(__attribute__((unused)) thread_t * thread)
+start_vrrp_termination_thread(__attribute__((unused)) thread_ref_t thread)
 {
 	/* This runs in the context of a thread */
 	two_phase_terminate = true;
@@ -434,7 +437,7 @@ stop_vrrp(int status)
 
 /* Daemon init sequence */
 static void
-start_vrrp(data_t *old_global_data)
+start_vrrp(data_t *prev_global_data)
 {
 	/* Clear the flags used for optimising performance */
 	clear_summary_flags();
@@ -443,7 +446,7 @@ start_vrrp(data_t *old_global_data)
 	if (!__test_bit(CONFIG_TEST_BIT, &debug))
 		kernel_netlink_init();
 
-	if (reload)
+	if (!global_data)
 		global_data = alloc_global_data();
 	else if (global_data->default_ifname) {
 		/* We need to set the default_ifp here on startup, since
@@ -462,6 +465,13 @@ start_vrrp(data_t *old_global_data)
 
 	init_data(conf_file, vrrp_init_keywords);
 
+	/* Update process name if necessary */
+	if ((!reload && global_data->vrrp_process_name) ||
+	    (reload &&
+	     (!global_data->vrrp_process_name != !prev_global_data->vrrp_process_name ||
+	      (global_data->vrrp_process_name && strcmp(global_data->vrrp_process_name, prev_global_data->vrrp_process_name)))))
+		set_process_name(global_data->vrrp_process_name);
+
 	if (non_existent_interface_specified) {
 		report_config_error(CONFIG_BAD_IF, "Non-existent interface specified in configuration");
 		stop_vrrp(KEEPALIVED_EXIT_CONFIG);
@@ -469,13 +479,13 @@ start_vrrp(data_t *old_global_data)
 	}
 
 	if (reload)
-		init_global_data(global_data, old_global_data);
+		init_global_data(global_data, prev_global_data, true);
 
 	/* Set our copy of time */
 	set_time_now();
 
 	if (!__test_bit(CONFIG_TEST_BIT, &debug)) {
-#if defined _WITH_SNMP_RFC || defined _WITH_SNMP_VRRP_
+#if defined _WITH_SNMP_RFC_ || defined _WITH_SNMP_VRRP_
 		if (!reload && (
 #ifdef _WITH_SNMP_VRRP_
 		     global_data->enable_snmp_vrrp ||
@@ -486,7 +496,7 @@ start_vrrp(data_t *old_global_data)
 #ifdef _WITH_SNMP_RFCV3_
 		     global_data->enable_snmp_rfcv3 ||
 #endif
-		     false)) {
+		     snmp_option)) {
 			vrrp_snmp_agent_init(global_data->snmp_socket);
 #ifdef _WITH_SNMP_RFC_
 			vrrp_start_time = time_now;
@@ -539,12 +549,6 @@ start_vrrp(data_t *old_global_data)
 			netlink_error_ignore = 0;
 #endif
 		}
-
-#ifdef _WITH_DBUS_
-		if (!reload && global_data->enable_dbus)
-			if (!dbus_start())
-				global_data->enable_dbus = false;
-#endif
 	}
 
 	/* Complete VRRP initialization */
@@ -567,16 +571,16 @@ start_vrrp(data_t *old_global_data)
 	else
 		ndisc_close();
 
+#ifdef _WITH_FIREWALL_
 	/* We need to delay the init of iptables to after vrrp_complete_init()
 	 * has been called so we know whether we want IPv4 and/or IPv6 */
-	iptables_init();
+	firewall_init();
 
 	/* Make sure we don't have any old iptables/ipsets settings left around */
-#ifdef _HAVE_LIBIPTC_
 	if (!reload)
-		iptables_cleanup();
+		firewall_cleanup();
 
-	iptables_startup(reload);
+	firewall_startup(reload);
 #endif
 
 	if (!reload)
@@ -584,12 +588,22 @@ start_vrrp(data_t *old_global_data)
 
 	/* clear_diff_vrrp must be called after vrrp_complete_init, since the latter
 	 * sets ifp on the addresses, which is used for the address comparison */
-	if (reload)
+	if (reload) {
 		clear_diff_vrrp();
+		vrrp_dispatcher_release(old_vrrp_data);
+	}
 
 #ifdef _WITH_DBUS_
-	if (reload && global_data->enable_dbus)
-		dbus_reload(old_vrrp_data->vrrp, vrrp_data->vrrp);
+	if (global_data->enable_dbus) {
+		if (reload && old_global_data->enable_dbus)
+			dbus_reload(old_vrrp_data->vrrp, vrrp_data->vrrp);
+		else {
+			if (!dbus_start())
+				global_data->enable_dbus = false;
+		}
+	}
+	else if (reload && old_global_data->enable_dbus)
+		dbus_stop();
 #endif
 
 	/* Post initializations */
@@ -609,8 +623,12 @@ start_vrrp(data_t *old_global_data)
 		dump_data_vrrp(NULL);
 
 	/* Init & start the VRRP packet dispatcher */
-	thread_add_event(master, vrrp_dispatcher_init, NULL,
-			 VRRP_DISPATCHER);
+	if (!reload && global_data->vrrp_startup_delay) {
+		log_message(LOG_INFO, "Delaying startup for %g seconds", global_data->vrrp_startup_delay / TIMER_HZ_DOUBLE);
+		thread_add_timer(master, vrrp_dispatcher_init, NULL,
+				 global_data->vrrp_startup_delay);
+	} else
+		thread_add_event(master, vrrp_dispatcher_init, NULL, 0);
 
 	/* Set the process priority and non swappable if configured */
 	set_process_priorities(
@@ -622,13 +640,18 @@ start_vrrp(data_t *old_global_data)
 #endif
 			       global_data->vrrp_process_priority, global_data->vrrp_no_swap ? 4096 : 0);
 
+#ifdef _HAVE_SCHED_RT_
+	/* Set the process cpu affinity if configured */
+	set_process_cpu_affinity(&global_data->vrrp_cpu_mask, "vrrp");
+#endif
+
 	/* Ensure we can open sufficient file descriptors */
 	set_vrrp_max_fds();
 }
 
 #ifndef _DEBUG_
 static int
-send_reload_advert_thread(thread_t *thread)
+send_reload_advert_thread(thread_ref_t thread)
 {
 	vrrp_t *vrrp = THREAD_ARG(thread);
 
@@ -724,8 +747,10 @@ vrrp_signal_init(void)
 
 /* Reload thread */
 static int
-reload_vrrp_thread(__attribute__((unused)) thread_t * thread)
+reload_vrrp_thread(__attribute__((unused)) thread_ref_t thread)
 {
+	bool with_snmp = false;
+
 	log_message(LOG_INFO, "Reloading");
 
 	/* Use standard scheduling while reloading */
@@ -742,10 +767,27 @@ reload_vrrp_thread(__attribute__((unused)) thread_t * thread)
 
 	vrrp_initialised = false;
 
+#if !defined _DEBUG_ && defined _WITH_SNMP_VRRP_
+	if (
+#ifdef _WITH_SNMP_VRRP_
+	    global_data->enable_snmp_vrrp ||
+#endif
+#ifdef _WITH_SNMP_RFCV2_
+	    global_data->enable_snmp_rfcv2 ||
+#endif
+#ifdef _WITH_SNMP_RFCV3_
+	    global_data->enable_snmp_rfcv3 ||
+#endif
+	    snmp_option)
+		with_snmp = true;
+#endif
+
 	/* Destroy master thread */
-	vrrp_dispatcher_release(vrrp_data);
+#ifdef _WITH_BFD_
+	cancel_vrrp_threads();
+#endif
 	thread_cleanup_master(master);
-	thread_add_base_threads(master);
+	thread_add_base_threads(master, with_snmp);
 
 #ifdef _WITH_LVS_
 	if (global_data->lvs_syncd.ifname)
@@ -798,14 +840,14 @@ reload_vrrp_thread(__attribute__((unused)) thread_t * thread)
 }
 
 static int
-print_vrrp_data(__attribute__((unused)) thread_t * thread)
+print_vrrp_data(__attribute__((unused)) thread_ref_t thread)
 {
 	vrrp_print_data();
 	return 0;
 }
 
 static int
-print_vrrp_stats(__attribute__((unused)) thread_t * thread)
+print_vrrp_stats(__attribute__((unused)) thread_ref_t thread)
 {
 	vrrp_print_stats();
 	return 0;
@@ -813,7 +855,7 @@ print_vrrp_stats(__attribute__((unused)) thread_t * thread)
 
 #ifdef _WITH_JSON_
 static int
-print_vrrp_json(__attribute__((unused)) thread_t * thread)
+print_vrrp_json(__attribute__((unused)) thread_ref_t thread)
 {
 	vrrp_print_json();
 	return 0;
@@ -822,7 +864,7 @@ print_vrrp_json(__attribute__((unused)) thread_t * thread)
 
 /* VRRP Child respawning thread */
 static int
-vrrp_respawn_thread(thread_t * thread)
+vrrp_respawn_thread(thread_ref_t thread)
 {
 	/* We catch a SIGCHLD, handle it */
 	vrrp_child = 0;
@@ -859,6 +901,9 @@ register_vrrp_thread_addresses(void)
 #endif
 	register_vrrp_fifo_addresses();
 	register_vrrp_inotify_addresses();
+#ifdef _WITH_CN_PROC_
+	register_process_monitor_addresses();
+#endif
 
 #ifndef _DEBUG_
 	register_thread_address("print_vrrp_data", print_vrrp_data);
@@ -888,7 +933,7 @@ start_vrrp_child(void)
 {
 #ifndef _DEBUG_
 	pid_t pid;
-	char *syslog_ident;
+	const char *syslog_ident;
 
 	/* Initialize child process */
 #ifdef ENABLE_LOG_TO_FILE
