@@ -53,8 +53,8 @@
 
 #include "logger.h"
 #include "global_data.h"
-#include "vrrp_iptables.h"
 #include "vrrp_ipset.h"
+#include "vrrp_firewall.h"
 #include "vrrp_iptables_calls.h"
 #include "main.h"
 
@@ -65,7 +65,7 @@
 #ifdef LIBIPSET_PRE_V7_COMPAT
 struct ipset_session* (*ipset_session_init_addr)(ipset_outfn outfn);
 #else
-struct ipset_session* (*ipset_session_init_addr)(ipset_print_outfn outfn, void *p);
+struct ipset_session* (*ipset_session_init_addr)(ipset_print_outfn outfn, const void *p);
 #endif
 int (*ipset_session_fini_addr)(struct ipset_session *session);
 struct ipset_data* (*ipset_session_data_addr)(const struct ipset_session *session);
@@ -101,8 +101,10 @@ static void* libipset_handle;
 
 static int
 #ifdef LIBIPSET_PRE_V7_COMPAT
+__attribute__ ((format(printf, 1, 2)))
 ipset_printf(const char *fmt, ...)
 #else
+__attribute__ ((format(printf, 3, 4)))
 ipset_printf(__attribute ((__unused__)) struct ipset_session *session, void *p, const char *fmt, ...)
 #endif
 {
@@ -190,7 +192,7 @@ has_ipset_setname(void* vsession, const char *setname)
 	return ipset_cmd1(session, IPSET_CMD_HEADER, 0) == 0;
 }
 
-static bool create_sets(const char* addr4, const char* addr6, const char* addr_if6, bool reload)
+static bool create_sets(const char* addr4, const char* addr6, const char* addr_if6, bool is_reload)
 {
 	struct ipset_session *session;
 
@@ -206,7 +208,7 @@ static bool create_sets(const char* addr4, const char* addr6, const char* addr_i
 
 	/* If we aren't reloading, don't worry if sets already exists. With the
 	 * IPSET_ENV_EXIST option set, any existing entries in the set are removed. */
-	if (!reload)
+	if (!is_reload)
 #ifdef LIBIPSET_PRE_V7_COMPAT
 		ipset_envopt_parse(session, IPSET_ENV_EXIST, NULL);
 #else
@@ -214,14 +216,14 @@ static bool create_sets(const char* addr4, const char* addr6, const char* addr_i
 #endif
 
 	if (block_ipv4) {
-		if (!reload || !has_ipset_setname(session, addr4))
+		if (!is_reload || !has_ipset_setname(session, addr4))
 			ipset_create(session, addr4, "hash:ip", NFPROTO_IPV4);
 	}
 
 	if (block_ipv6) {
-		if (!reload || !has_ipset_setname(session, addr6))
+		if (!is_reload || !has_ipset_setname(session, addr6))
 			ipset_create(session, addr6, "hash:ip", NFPROTO_IPV6);
-		if (!reload || !has_ipset_setname(session, addr_if6)) {
+		if (!is_reload || !has_ipset_setname(session, addr_if6)) {
 #ifdef HAVE_IPSET_ATTR_IFACE
 			/* hash:net,iface was introduced in Linux 3.1 */
 			ipset_create(session, addr_if6, "hash:net,iface", NFPROTO_IPV6);
@@ -273,7 +275,7 @@ bool ipset_initialise(void)
 	 * than Linux 3.13, since ipsets didn't understand namespaces prior to that. */
 	if (global_data->network_namespace &&
 	    !global_data->namespace_with_ipsets &&
-	    !strcmp(global_data->vrrp_ipset_address, "keepalived") &&
+	    !strcmp(global_data->vrrp_ipset_address, DEFAULT_IPSET_NAME) &&
 	    (os_major <= 2 ||
 	     (os_major == 3 && os_minor < 13))) {
 		log_message(LOG_INFO, "Not using ipsets with network namespace since not supported with kernel version < 3.13");
@@ -351,9 +353,9 @@ bool remove_ipsets(void)
 	return true;
 }
 
-bool add_ipsets(bool reload)
+bool add_ipsets(bool is_reload)
 {
-	return create_sets(global_data->vrrp_ipset_address, global_data->vrrp_ipset_address6, global_data->vrrp_ipset_address_iface6, reload);
+	return create_sets(global_data->vrrp_ipset_address, global_data->vrrp_ipset_address6, global_data->vrrp_ipset_address_iface6, is_reload);
 }
 
 void* ipset_session_start(void)
