@@ -17,7 +17,7 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2016 Alexandre Cassen, <acassen@gmail.com>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #include "config.h"
@@ -38,11 +38,16 @@
 #include <libiptc/libiptc.h>
 #include <libiptc/libip6tc.h>
 #ifdef _HAVE_LIBIPSET_
+#ifdef USE_LIBIPSET_LINUX_IP_SET_H
+#include <libipset/linux_ip_set.h>
+#else
 #include <linux/netfilter/ipset/ip_set.h>
+#endif
 #include <linux/netfilter/xt_set.h>
 #endif
 #include <unistd.h>
 #include <signal.h>
+#include <errno.h>
 #include <stdint.h>
 
 #include "vrrp_iptables_calls.h"
@@ -54,7 +59,9 @@
 #ifdef _LIBIPTC_DYNAMIC_
 #include "global_data.h"
 #endif
-#include "vrrp_iptables.h"
+#include "vrrp_iptables_lib.h"
+#include "vrrp_firewall.h"
+#include "utils.h"
 
 /* We sometimes get a resource_busy on iptc_commit. This appears to happen
  * when someone else is also updating it.
@@ -225,7 +232,7 @@ int ip4tables_process_entry( struct iptc_handle* handle, const char* chain_name,
 		if ( protocol == IPPROTO_ICMP )
 		{
 			match = (struct xt_entry_match*)((char*)fw + fw->target_offset);
-			match->u.match_size = XT_ALIGN(sizeof (struct xt_entry_match)) + XT_ALIGN(sizeof(struct ipt_icmp));  
+			match->u.match_size = XT_ALIGN(sizeof (struct xt_entry_match)) + XT_ALIGN(sizeof(struct ipt_icmp));
 			match->u.user.revision = 0;
 			fw->target_offset = (uint16_t)(fw->target_offset + match->u.match_size);
 			strcpy ( match->u.user.name, "icmp" ) ;
@@ -240,16 +247,17 @@ int ip4tables_process_entry( struct iptc_handle* handle, const char* chain_name,
 
 // target is XTC_LABEL_DROP/XTC_LABEL_ACCEPT
 	fw->next_offset = (uint16_t)size;
-	target = ipt_get_target ( fw ) ;
+	target = ipt_get_target(fw);
 	target->u.user.target_size = XT_ALIGN (sizeof (struct xt_entry_target) + 1);
-	strcpy (target->u.user.name, target_name );
+	strcpy_safe(target->u.user.name, target_name);
 //	fw->ip.flags |= IPT_F_GOTO;
-	strcpy (chain, chain_name);
+	strcpy_safe(chain, chain_name);
 	// Use iptc_append_entry to add to the chain
 	if (cmd == IPADDRESS_DEL) {
-		unsigned char matchmask[fw->next_offset];
+		unsigned char *matchmask = MALLOC(fw->next_offset);
 		memset(matchmask, 0xff, fw->next_offset);
 		res = iptc_delete_entry(chain, fw, matchmask, handle);
+		FREE(matchmask);
 	}
 	else if (rulenum == APPEND_RULE)
 		res = iptc_append_entry (chain, fw, handle ) ;
@@ -371,15 +379,16 @@ int ip6tables_process_entry( struct ip6tc_handle* handle, const char* chain_name
 	fw->next_offset = (uint16_t)size;
 	target = ip6t_get_target ( fw ) ;
 	target->u.user.target_size = XT_ALIGN (sizeof (struct xt_entry_target) + 1);
-	strcpy (target->u.user.name, target_name );
+	strcpy_safe(target->u.user.name, target_name);
 //	fw->ip.flags |= IPT_F_GOTO;
-	strcpy (chain, chain_name);
+	strcpy_safe(chain, chain_name);
 
 	// Use iptc_append_entry to add to the chain
 	if (cmd == IPADDRESS_DEL) {
-		unsigned char matchmask[fw->next_offset];
+		unsigned char *matchmask = MALLOC(fw->next_offset);
 		memset(matchmask, 0xff, fw->next_offset);
 		res = ip6tc_delete_entry ( chain, fw, matchmask, handle);
+		FREE(matchmask);
 	}
 	else if (rulenum == APPEND_RULE)
 		res = ip6tc_append_entry (chain, fw, handle ) ;
@@ -401,7 +410,7 @@ int ip6tables_process_entry( struct ip6tc_handle* handle, const char* chain_name
 }
 
 #ifdef _HAVE_LIBIPTC_
-#ifdef _LIBXTABLES_DYNAMIC_ 
+#ifdef _LIBXTABLES_DYNAMIC_
 static
 bool xtables_load(void)
 {
@@ -484,9 +493,6 @@ struct ip_set_req_get_set {
  union ip_set_name_index set;
 };
 
-#define IP_SET_OP_GET_BYINDEX 0x00000007 /* Get set name by index */
-/* Uses ip_set_req_get_set */
-
 #define IP_SET_OP_VERSION 0x00000100 /* Ask kernel version */
 struct ip_set_req_version {
  unsigned op;
@@ -502,8 +508,10 @@ get_version(unsigned int* version)
 	socklen_t size = sizeof(req_version);
 	int res;
 
-	if (sockfd < 0)
+	if (sockfd < 0) {
 		log_message(LOG_INFO, "Can't open socket to ipset.");
+		return -1;
+	}
 
 #if !HAVE_DECL_SOCK_CLOEXEC
 	if (fcntl(sockfd, F_SETFD, FD_CLOEXEC) == -1) {
@@ -569,7 +577,11 @@ get_set_byname(const char *setname, struct xt_set_info *info, unsigned family, b
 
 	info->index = IPSET_INVALID_ID;
 
-	sockfd = get_version(&version);
+	if ((sockfd = get_version(&version)) == -1) {
+		info->index = IPSET_INVALID_ID;
+		return;
+	}
+
 #if defined IP_SET_OP_GET_FNAME		/* Since Linux 3.13 */
 	req.version = version;
 	req.op = IP_SET_OP_GET_FNAME;
@@ -638,9 +650,6 @@ int ip4tables_add_rules(struct iptc_handle* handle, const char* chain_name, unsi
 	int sav_errno;
 
 	/* Add an entry */
-
-	memset(chain, 0, sizeof(chain));
-
 	size = XT_ALIGN(sizeof (struct ipt_entry)) +
 			XT_ALIGN(sizeof(struct xt_entry_match)) +
 			XT_ALIGN(sizeof(struct xt_entry_target) + 1) +
@@ -713,15 +722,16 @@ int ip4tables_add_rules(struct iptc_handle* handle, const char* chain_name, unsi
 	fw->next_offset = (uint16_t)size;
 	target = ipt_get_target(fw);
 	target->u.user.target_size = XT_ALIGN(sizeof(struct xt_entry_target) + 1);
-	strcpy(target->u.user.name, target_name);
+	strcpy_safe(target->u.user.name, target_name);
 //	fw->ip.flags |= IPT_F_GOTO;
-	strcpy(chain, chain_name);
+	strcpy_safe(chain, chain_name);
 
 	// Use iptc_append_entry to add to the chain
 	if (cmd == IPADDRESS_DEL) {
-		unsigned char matchmask[fw->next_offset];
+		unsigned char *matchmask = MALLOC(fw->next_offset);
 		memset(matchmask, 0xff, fw->next_offset);
 		res = iptc_delete_entry(chain, fw, matchmask, handle);
+		FREE(matchmask);
 	}
 	else if (rulenum == APPEND_RULE)
 		res = iptc_append_entry(chain, fw, handle) ;
@@ -735,7 +745,7 @@ int ip4tables_add_rules(struct iptc_handle* handle, const char* chain_name, unsi
 	if (res!= 1)
 	{
 		if (!ignore_errors)
-			log_message(LOG_INFO, "iptc_insert_entry for chain %s returned %d: %s", chain, res, iptc_strerror(sav_errno)) ;
+			log_message(LOG_INFO, "iptc_insert_entry for chain %s returned %d: %s", chain_name, res, iptc_strerror(sav_errno)) ;
 
 		return sav_errno;
 	}
@@ -838,15 +848,16 @@ int ip6tables_add_rules(struct ip6tc_handle* handle, const char* chain_name, uns
 	fw->next_offset = (uint16_t)size;
 	target = ip6t_get_target(fw);
 	target->u.user.target_size = XT_ALIGN(sizeof(struct xt_entry_target) + 1);
-	strcpy(target->u.user.name, target_name);
+	strcpy_safe(target->u.user.name, target_name);
 //	fw->ip.flags |= IP6T_F_GOTO;
-	strcpy(chain, chain_name);
+	strcpy_safe(chain, chain_name);
 
 	// Use iptc_append_entry to add to the chain
 	if (cmd == IPADDRESS_DEL) {
-		unsigned char matchmask[fw->next_offset];
+		unsigned char *matchmask = MALLOC(fw->next_offset);
 		memset(matchmask, 0xff, fw->next_offset);
 		res = ip6tc_delete_entry(chain, fw, matchmask, handle);
+		FREE(matchmask);
 	}
 	else if (rulenum == APPEND_RULE)
 		res = ip6tc_append_entry(chain, fw, handle) ;
