@@ -33,6 +33,7 @@
 #include <sys/stat.h>
 #include <stdint.h>
 #include <errno.h>
+#include <sys/prctl.h>
 #ifdef _WITH_PERF_
 #include <stdio.h>
 #include <sys/types.h>
@@ -66,10 +67,11 @@
 
 /* global vars */
 unsigned long debug = 0;
+mode_t umask_val = S_IXUSR | S_IRWXG | S_IRWXO;
 
 /* Display a buffer into a HEXA formated output */
 void
-dump_buffer(char *buff, size_t count, FILE* fp, int indent)
+dump_buffer(const char *buff, size_t count, FILE* fp, int indent)
 {
 	size_t i, j, c;
 	bool printnext = true;
@@ -85,7 +87,7 @@ dump_buffer(char *buff, size_t count, FILE* fp, int indent)
 			fprintf(fp, "%*s%.4zu ", indent, "", i & 0xffff);
 		}
 		if (i < count)
-			fprintf(fp, "%3.2x", buff[i] & 0xff);
+			fprintf(fp, "%3.2x", (unsigned char)buff[i] & 0xff);
 		else
 			fprintf(fp, "   ");
 		if (!((i + 1) % 8)) {
@@ -110,14 +112,41 @@ dump_buffer(char *buff, size_t count, FILE* fp, int indent)
 	}
 }
 
+#ifdef CHECKSUM_DIAGNOSTICS
+void
+log_buffer(const char *msg, const void *buff, size_t count)
+{
+	char op_buf[60];	// Probably 56 really
+	const unsigned char *bufp = buff;
+	char *ptr;
+	size_t offs = 0;
+	unsigned i;
+
+	log_message(LOG_INFO, "%s - len %zu", msg, count);
+
+	while (offs < count) {
+		ptr = op_buf;
+		ptr += snprintf(ptr, op_buf + sizeof(op_buf) - ptr, "%4.4lx ", offs);
+
+		for (i = 0; i < 16 && offs < count; i++) {
+			if (i == 8)
+				*ptr++ = ' ';
+			ptr += snprintf(ptr, op_buf + sizeof(op_buf) - ptr, " %2.2x", bufp[offs++]);
+		}
+
+		log_message(LOG_INFO, "%s", op_buf);
+	}
+}
+#endif
+
 #ifdef _WITH_STACKTRACE_
 void
 write_stacktrace(const char *file_name, const char *str)
 {
 	int fd;
 	void *buffer[100];
-	int nptrs;
-	int i;
+	unsigned int nptrs;
+	unsigned int i;
 	char **strs;
 
 	nptrs = backtrace(buffer, 100);
@@ -140,14 +169,15 @@ write_stacktrace(const char *file_name, const char *str)
 		}
 
 		/* We don't need the call to this function, or the first two entries on the stack */
-		for (i = 1; i < nptrs - 2; i++)
+		nptrs -= 2;
+		for (i = 1; i < nptrs; i++)
 			log_message(LOG_INFO, "  %s", strs[i]);
 		free(strs);
 	}
 }
 #endif
 
-char *
+const char *
 make_file_name(const char *name, const char *prog, const char *namespace, const char *instance)
 {
 	const char *extn_start;
@@ -189,6 +219,16 @@ make_file_name(const char *name, const char *prog, const char *namespace, const
 	return file_name;
 }
 
+void
+set_process_name(const char *name)
+{
+	if (!name)
+		name = "keepalived";
+
+	if (prctl(PR_SET_NAME, name))
+		log_message(LOG_INFO, "Failed to set process name '%s'", name);
+}
+
 #ifdef _WITH_PERF_
 void
 run_perf(const char *process, const char *network_namespace, const char *instance_name)
@@ -196,7 +236,7 @@ run_perf(const char *process, const char *network_namespace, const char *instanc
 	int ret;
 	pid_t pid;
 	char *orig_name = NULL;
-	char *new_name;
+	const char *new_name;
 	const char *perf_name = "perf.data";
 	int in = -1;
 	int ep = -1;
@@ -264,16 +304,18 @@ run_perf(const char *process, const char *network_namespace, const char *instanc
 				break;
 			}
 			else if (ret == -1) {
-				if (errno == EINTR)
+				if (check_EINTR(errno))
 					continue;
+
 				log_message(LOG_INFO, "perf epoll returned errno %d - %m", errno);
 				break;
 			}
 
 			ret = read(in, buf, sizeof(buf));
 			if (ret == -1) {
-				if (errno == EINTR)
+				if (check_EINTR(errno))
 					continue;
+
 				log_message(LOG_INFO, "perf inotify read returned errno %d %m", errno);
 				break;
 			}
@@ -306,7 +348,7 @@ run_perf(const char *process, const char *network_namespace, const char *instanc
 			if (rename(orig_name, new_name))
 				log_message(LOG_INFO, "Rename %s to %s failed - %m (%d)", orig_name, new_name, errno);
 
-			FREE(new_name);
+			FREE_CONST(new_name);
 		} while (false);
 	} while (false);
 
@@ -341,7 +383,7 @@ in_csum(const uint16_t *addr, size_t len, uint32_t csum, uint32_t *acc)
 
 	/* mop up an odd byte, if necessary */
 	if (nleft == 1)
-		sum += htons(*(u_char *) w << 8);
+		sum += htons(*(const u_char *)w << 8);
 
 	if (acc)
 		*acc = sum;
@@ -356,13 +398,13 @@ in_csum(const uint16_t *addr, size_t len, uint32_t csum, uint32_t *acc)
 }
 
 /* IP network to ascii representation */
-char *
+const char *
 inet_ntop2(uint32_t ip)
 {
 	static char buf[16];
-	unsigned char *bytep;
+	const unsigned char *bytep;
 
-	bytep = (unsigned char *) &(ip);
+	bytep = (const unsigned char *)&ip;
 	sprintf(buf, "%d.%d.%d.%d", bytep[0], bytep[1], bytep[2], bytep[3]);
 	return buf;
 }
@@ -375,9 +417,9 @@ inet_ntop2(uint32_t ip)
 char *
 inet_ntoa2(uint32_t ip, char *buf)
 {
-	unsigned char *bytep;
+	const unsigned char *bytep;
 
-	bytep = (unsigned char *) &(ip);
+	bytep = (const unsigned char *)&ip;
 	sprintf(buf, "%d.%d.%d.%d", bytep[0], bytep[1], bytep[2], bytep[3]);
 	return buf;
 }
@@ -391,7 +433,7 @@ inet_stor(const char *addr, uint32_t *range_end)
 	char *endptr;
 	unsigned long range;
 	int family = strchr(addr, ':') ? AF_INET6 : AF_INET;
-	char *warn = "";
+	const char *warn = "";
 
 #ifndef _STRICT_CONFIG_
 	if (!__test_bit(CONFIG_TEST_BIT, &debug))
@@ -466,13 +508,14 @@ domain_stosockaddr(const char *domain, const char *port, struct sockaddr_storage
 	return 0;
 }
 
-/* IP string to sockaddr_storage */
-int
-inet_stosockaddr(char *ip, const char *port, struct sockaddr_storage *addr)
+/* IP string to sockaddr_storage
+ *   return value is "error". */
+bool
+inet_stosockaddr(const char *ip, const char *port, struct sockaddr_storage *addr)
 {
 	void *addr_ip;
-	char *cp;
-	char sav_cp;
+	const char *cp;
+	char *ip_str = NULL;
 	unsigned port_num;
 	int res;
 
@@ -481,7 +524,7 @@ inet_stosockaddr(char *ip, const char *port, struct sockaddr_storage *addr)
 	if (port) {
 		if (!read_unsigned(port, &port_num, 1, 65535, true)) {
 			addr->ss_family = AF_UNSPEC;
-			return -1;
+			return true;
 		}
 	}
 
@@ -499,28 +542,25 @@ inet_stosockaddr(char *ip, const char *port, struct sockaddr_storage *addr)
 
 	/* remove range and mask stuff */
 	if ((cp = strchr(ip, '-')) ||
-	    (cp = strchr(ip, '/'))) {
-		sav_cp = *cp;
-		*cp = 0;
-	}
+	    (cp = strchr(ip, '/')))
+		ip_str = STRNDUP(ip, cp - ip);
 
-	res = inet_pton(addr->ss_family, ip, addr_ip);
+	res = inet_pton(addr->ss_family, ip_str ? ip_str : ip, addr_ip);
 
-	/* restore range and mask stuff */
-	if (cp)
-		*cp = sav_cp;
+	if (ip_str)
+		free(ip_str);
 
 	if (!res) {
 		addr->ss_family = AF_UNSPEC;
-		return -1;
+		return true;
 	}
 
-	return 0;
+	return false;
 }
 
 /* IPv4 to sockaddr_storage */
 void
-inet_ip4tosockaddr(struct in_addr *sin_addr, struct sockaddr_storage *addr)
+inet_ip4tosockaddr(const struct in_addr *sin_addr, struct sockaddr_storage *addr)
 {
 	struct sockaddr_in *addr4 = (struct sockaddr_in *) addr;
 	addr4->sin_family = AF_INET;
@@ -529,24 +569,66 @@ inet_ip4tosockaddr(struct in_addr *sin_addr, struct sockaddr_storage *addr)
 
 /* IPv6 to sockaddr_storage */
 void
-inet_ip6tosockaddr(struct in6_addr *sin_addr, struct sockaddr_storage *addr)
+inet_ip6tosockaddr(const struct in6_addr *sin_addr, struct sockaddr_storage *addr)
 {
 	struct sockaddr_in6 *addr6 = (struct sockaddr_in6 *) addr;
 	addr6->sin6_family = AF_INET6;
 	addr6->sin6_addr = *sin_addr;
 }
 
+/* Check address, possibly with mask, is valid */
+bool
+check_valid_ipaddress(const char *str, bool allow_subnet_mask)
+{
+	int family;
+	unsigned long prefixlen;
+	const char *p;
+	char *endptr;
+	union {
+		struct in_addr in;
+		struct in6_addr in6;
+	} addr;
+	int res;
+	const char *str_dup = NULL;
+
+	if (!strchr(str, ':') && !strchr(str, '.'))
+		return false;
+
+	family = (strchr(str, ':')) ? AF_INET6 : AF_INET;
+
+	if (allow_subnet_mask)
+		p = strchr(str, '/');
+	else
+		p = NULL;
+
+	if (p) {
+		if (!p[1])
+			return false;
+		prefixlen = strtoul(p + 1, &endptr, 10);
+		if (*endptr || prefixlen > (family == AF_INET6 ? 128 : 32))
+			return false;
+		str_dup = STRNDUP(str, p - str);
+	}
+
+	res = inet_pton(family, str_dup ? str_dup : str, &addr);
+
+	if (str_dup)
+		FREE_CONST(str_dup);
+
+	return res;
+}
+
 /* IP network to string representation */
 static char *
-inet_sockaddrtos2(struct sockaddr_storage *addr, char *addr_str)
+inet_sockaddrtos2(const struct sockaddr_storage *addr, char *addr_str)
 {
-	void *addr_ip;
+	const void *addr_ip;
 
 	if (addr->ss_family == AF_INET6) {
-		struct sockaddr_in6 *addr6 = (struct sockaddr_in6 *) addr;
+		const struct sockaddr_in6 *addr6 = (const struct sockaddr_in6 *) addr;
 		addr_ip = &addr6->sin6_addr;
 	} else {
-		struct sockaddr_in *addr4 = (struct sockaddr_in *) addr;
+		const struct sockaddr_in *addr4 = (const struct sockaddr_in *) addr;
 		addr_ip = &addr4->sin_addr;
 	}
 
@@ -556,30 +638,42 @@ inet_sockaddrtos2(struct sockaddr_storage *addr, char *addr_str)
 	return addr_str;
 }
 
-char *
-inet_sockaddrtos(struct sockaddr_storage *addr)
+const char *
+inet_sockaddrtos(const struct sockaddr_storage *addr)
 {
 	static char addr_str[INET6_ADDRSTRLEN];
 	inet_sockaddrtos2(addr, addr_str);
 	return addr_str;
 }
 
-uint16_t
-inet_sockaddrport(struct sockaddr_storage *addr)
+uint16_t __attribute__ ((pure))
+inet_sockaddrport(const struct sockaddr_storage *addr)
 {
 	if (addr->ss_family == AF_INET6) {
-		struct sockaddr_in6 *addr6 = (struct sockaddr_in6 *) addr;
+		const struct sockaddr_in6 *addr6 = (const struct sockaddr_in6 *) addr;
 		return addr6->sin6_port;
 	}
 
 	/* Note: this might be AF_UNSPEC if it is the sequence number of
 	 * a virtual server in a virtual server group */
-	struct sockaddr_in *addr4 = (struct sockaddr_in *) addr;
+	const struct sockaddr_in *addr4 = (const struct sockaddr_in *) addr;
 	return addr4->sin_port;
 }
 
-char *
-inet_sockaddrtopair(struct sockaddr_storage *addr)
+void
+inet_set_sockaddrport(struct sockaddr_storage *addr, uint16_t port)
+{
+	if (addr->ss_family == AF_INET6) {
+		struct sockaddr_in6 *addr6 = (struct sockaddr_in6 *) addr;
+		addr6->sin6_port = port;
+	} else {
+		struct sockaddr_in *addr4 = (struct sockaddr_in *) addr;
+		addr4->sin_port = port;
+	}
+}
+
+const char *
+inet_sockaddrtopair(const struct sockaddr_storage *addr)
 {
 	char addr_str[INET6_ADDRSTRLEN];
 	static char ret[sizeof(addr_str) + 8];	/* '[' + addr_str + ']' + ':' + 'nnnnn' */
@@ -592,39 +686,52 @@ inet_sockaddrtopair(struct sockaddr_storage *addr)
 }
 
 char *
-inet_sockaddrtotrio(struct sockaddr_storage *addr, uint16_t proto)
+inet_sockaddrtotrio_r(const struct sockaddr_storage *addr, uint16_t proto, char *buf)
 {
 	char addr_str[INET6_ADDRSTRLEN];
-	static char ret[sizeof(addr_str) + 13];	/* '[' + addr_str + ']' + ':' + 'sctp' + ':' + 'nnnnn' */
-	char *proto_str = proto == IPPROTO_TCP ? "tcp" : proto == IPPROTO_UDP ? "udp" : proto == IPPROTO_SCTP ? "sctp" : proto == 0 ? "none" : "?";
+	const char *proto_str =
+			proto == IPPROTO_TCP ? "tcp" :
+			proto == IPPROTO_UDP ? "udp" :
+			proto == IPPROTO_SCTP ? "sctp" :
+			proto == 0 ? "none" : "?";
 
 	inet_sockaddrtos2(addr, addr_str);
-	snprintf(ret, sizeof(ret), "[%s]:%s:%d" ,addr_str, proto_str,
+	snprintf(buf, SOCKADDRTRIO_STR_LEN, "[%s]:%s:%d", addr_str, proto_str,
 		 ntohs(inet_sockaddrport(addr)));
+	return buf;
+}
+
+const char *
+inet_sockaddrtotrio(const struct sockaddr_storage *addr, uint16_t proto)
+{
+	static char ret[SOCKADDRTRIO_STR_LEN];
+
+	inet_sockaddrtotrio_r(addr, proto, ret);
+
 	return ret;
 }
 
-uint32_t
-inet_sockaddrip4(struct sockaddr_storage *addr)
+uint32_t __attribute__ ((pure))
+inet_sockaddrip4(const struct sockaddr_storage *addr)
 {
 	if (addr->ss_family != AF_INET)
 		return 0xffffffff;
 
-	return ((struct sockaddr_in *) addr)->sin_addr.s_addr;
+	return ((const struct sockaddr_in *) addr)->sin_addr.s_addr;
 }
 
 int
-inet_sockaddrip6(struct sockaddr_storage *addr, struct in6_addr *ip6)
+inet_sockaddrip6(const struct sockaddr_storage *addr, struct in6_addr *ip6)
 {
 	if (addr->ss_family != AF_INET6)
 		return -1;
 
-	*ip6 = ((struct sockaddr_in6 *) addr)->sin6_addr;
+	*ip6 = ((const struct sockaddr_in6 *) addr)->sin6_addr;
 	return 0;
 }
 
 /* IPv6 address compare */
-int
+int __attribute__ ((pure))
 inet_inaddrcmp(const int family, const void *a, const void *b)
 {
 	int64_t addr_diff;
@@ -654,7 +761,7 @@ inet_inaddrcmp(const int family, const void *a, const void *b)
 	return -2;
 }
 
-int
+int  __attribute__ ((pure))
 inet_sockaddrcmp(const struct sockaddr_storage *a, const struct sockaddr_storage *b)
 {
 	if (a->ss_family != b->ss_family)
@@ -662,12 +769,12 @@ inet_sockaddrcmp(const struct sockaddr_storage *a, const struct sockaddr_storage
 
 	if (a->ss_family == AF_INET)
 		return inet_inaddrcmp(a->ss_family,
-				      &((struct sockaddr_in *) a)->sin_addr,
-				      &((struct sockaddr_in *) b)->sin_addr);
+				      &((const struct sockaddr_in *) a)->sin_addr,
+				      &((const struct sockaddr_in *) b)->sin_addr);
 	if (a->ss_family == AF_INET6)
 		return inet_inaddrcmp(a->ss_family,
-				      &((struct sockaddr_in6 *) a)->sin6_addr,
-				      &((struct sockaddr_in6 *) b)->sin6_addr);
+				      &((const struct sockaddr_in6 *) a)->sin6_addr,
+				      &((const struct sockaddr_in6 *) b)->sin6_addr);
 	return 0;
 }
 
@@ -678,7 +785,7 @@ inet_sockaddrcmp(const struct sockaddr_storage *a, const struct sockaddr_storage
  * Highly inspired from Paul Vixie code.
  */
 int
-inet_ston(const char *addr, uint32_t * dst)
+inet_ston(const char *addr, uint32_t *dst)
 {
 	static char digits[] = "0123456789";
 	int saw_digit, octets, ch;
@@ -740,14 +847,24 @@ inet_cidrtomask(uint8_t cidr)
 }
 #endif
 
+void
+format_mac_buf(char *op, size_t op_len, const unsigned char *addr, size_t addr_len)
+{
+	size_t i;
+	char *buf_end = op + op_len;
+
+	for (i = 0; i < addr_len; i++)
+		op += snprintf(op, buf_end - op, "%.2x%s",
+		      addr[i], i < addr_len -1 ? ":" : "");
+}
+
 /* Getting localhost official canonical name */
-char *
+const char * __attribute__((malloc))
 get_local_name(void)
 {
 	struct utsname name;
 	struct addrinfo hints, *res = NULL;
 	char *canonname = NULL;
-	size_t len = 0;
 
 	memset(&hints, 0, sizeof(struct addrinfo));
 	hints.ai_flags = AI_CANONNAME;
@@ -758,13 +875,8 @@ get_local_name(void)
 	if (getaddrinfo(name.nodename, NULL, &hints, &res) != 0)
 		return NULL;
 
-	if (res && res->ai_canonname) {
-		len = strlen(res->ai_canonname);
-		canonname = MALLOC(len + 1);
-		if (canonname) {
-			memcpy(canonname, res->ai_canonname, len);
-		}
-	}
+	if (res && res->ai_canonname)
+		canonname = STRDUP(res->ai_canonname);
 
 	freeaddrinfo(res);
 
@@ -772,7 +884,7 @@ get_local_name(void)
 }
 
 /* String compare with NULL string handling */
-bool
+bool __attribute__ ((pure))
 string_equal(const char *str1, const char *str2)
 {
 	if (!str1 && !str2)
@@ -783,15 +895,34 @@ string_equal(const char *str1, const char *str2)
 	return !strcmp(str1, str2);
 }
 
+/* Convert an integer into a string */
+int
+integer_to_string(const int value, char *str, size_t size)
+{
+	int i, len = 0, t = value, s = size;
+
+	for (i = value; i; i/=10) {
+		if (++len > s)
+			return -1;
+	}
+
+	for (i = 0; i < len; i++,t/=10)
+		str[len - (i + 1)] = t % 10 + '0';
+
+	return len;
+}
+
 /* We need to use O_NOFOLLOW if opening a file for write, so that a non privileged user can't
  * create a symbolic link from the path to a system file and cause a system file to be overwritten. */
 FILE *fopen_safe(const char *path, const char *mode)
 {
 	int fd;
 	FILE *file;
+#ifdef ENABLE_LOG_FILE_APPEND
 	int flags = O_NOFOLLOW | O_CREAT | O_CLOEXEC;
+#endif
 	int sav_errno;
-	char file_tmp_name[] = "/tmp/keepalivedXXXXXX";
+	char file_tmp_name[PATH_MAX];
 
 	if (mode[0] == 'r')
 		return fopen(path, mode);
@@ -811,11 +942,16 @@ FILE *fopen_safe(const char *path, const char *mode)
 		 * it leaves a window for someone else to create the same file between the unlink and the open.
 		 *
 		 * The solution is to create a temporary file that we will rename to the desired file name.
-		 * Since the temporary file is created owned by root with the only file access persmissions being
+		 * Since the temporary file is created owned by root with the only file access permissions being
 		 * owner read and write, no non root user will have access to the file. Further, the rename to
 		 * the requested filename is atomic, and so there is no window when someone else could create
 		 * another file of the same name.
 		 */
+		strcpy_safe(file_tmp_name, path);
+		if (strlen(path) + 6 < sizeof(file_tmp_name))
+			strcat(file_tmp_name, "XXXXXX");
+		else
+			strcpy(file_tmp_name + sizeof(file_tmp_name) - 6 - 1, "XXXXXX");
 		fd = mkostemp(file_tmp_name, O_CLOEXEC);
 	} else {
 		/* Only allow append mode if debugging features requiring append are enabled. Since we
@@ -856,8 +992,8 @@ FILE *fopen_safe(const char *path, const char *mode)
 	}
 #endif
 
-	/* Set file mode to rw------- */
-	if (fchmod(fd, S_IRUSR | S_IWUSR)) {
+	/* Set file mode, default rw------- */
+	if (fchmod(fd, (S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH) & ~umask_val)) {
 		sav_errno = errno;
 		log_message(LOG_INFO, "Unable to change file permission of %s - errno %d (%m)", path, errno);
 		close(fd);
@@ -905,6 +1041,8 @@ set_std_fd(bool force)
 	}
 
 	signal_fd_close(STDERR_FILENO+1);
+
+	/* coverity[leaked_handle] */
 }
 
 void
@@ -917,12 +1055,13 @@ close_std_fd(void)
 
 #if !defined _HAVE_LIBIPTC_ || defined _LIBIPTC_DYNAMIC_
 int
-fork_exec(char **argv)
+fork_exec(const char * const argv[])
 {
 	pid_t pid;
 	int status;
 	struct sigaction act, old_act;
 	int res = 0;
+	union non_const_args args;
 
 	act.sa_handler = SIG_DFL;
 	sigemptyset(&act.sa_mask);
@@ -944,7 +1083,8 @@ fork_exec(char **argv)
 
 		signal_handler_script();
 
-		execvp(*argv, argv);
+		args.args = argv;       /* Note: we are casting away constness, since execvp parameter type is wrong */
+		execvp(*argv, args.execve_args);
 		exit(EXIT_FAILURE);
 	} else {
 		/* Parent */
